/**
 * Webアプリケーション自動テストユーティリティ。
 * Webアプリケーションを自動的にテストするための、ユーティリティです。
 * このライブラリは、javaアプレットを使用します。
 *
 * Version 0.9.0.0
 *
 * このプログラムのライセンスについて。
 * このプログラムは、MITライセンス(The MIT License)の元で公開します。
 * Released under the MIT Licenses.
 *
 * ライセンスの原文は、以下のURLを参照してください。
 * http://www.yscjp.com/licenses/MIT_LICENSE.txt
 *
 * Copyright (c) 2010-2012 Yagyu Teru, http://yscjp.com/
 *
 */
var com;
if (!com) com = {};
if (!com.yscjp) com.yscjp = {};
if (!com.yscjp.webtools) com.yscjp.webtools = {};

/**
 * appletで初期化したJSRobotクラスのインスタンスを作成します。
 * @class Javaのjava.awt.Robotを使用して、ネイティブのイベントを発生させます。
 * @param applet applet要素
 * @param isAsyncKey キー操作を非同期で行います。IE７以上では、非同期にキー操作を行わなければ、
 * 正常にキー入力が（なぜか）できない為です。
 */
com.yscjp.webtools.JSRobot = function(applet, isAsyncKey) {

    var ArgChk = com.yscjp.jsapp.util.ArgChk;
    ArgChk.notNull(applet);
    isAsyncKey = ArgChk.notNull(isAsyncKey, false);

    /**
     * アプレット
     * @type Element
     */
    this.applet = applet;
    this.isAsyncKey = isAsyncKey;
    this.isOverScreen = false;
};

(function() {

    var JSHtmlUtil = com.yscjp.jsapp.util.JSHtmlUtil;
    var HtmlPosition = com.yscjp.jsapp.tools.HtmlPosition;
    var Log = com.yscjp.jsapp.util.Log;

    /**
     * JSRobotクラス
     */
    var JSRobot = com.yscjp.webtools.JSRobot;

    /*
     * マウスボタンを押下するときのボタンのマスク
     * Javaのイベントに合わせてあります。
     * mousePress と mouseRelease で使用します。
     */
    JSRobot.MOUSE_BUTTON1 = 16;
    JSRobot.MOUSE_BUTTON2 = 8;
    JSRobot.MOUSE_BUTTON3 = 4;

    /**
     * テストを行うターゲットのWindowを設定します。
     * @param targetWindow ターゲットのWindowオブジェクト
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.setDocument = function (targetWindow) {

        this.targetWindow = targetWindow;
        if (null != this.targetWindow) {

            this.targetDocument = this.targetWindow.document;

        } else {

            this.targetDocument = document;
        }

        this.documentTopOnScreen = 0;
        this.documentLeftOnScreen = 0;
    };
    //this.setDocument(targetWindow);

    /**
     * テストを行うターゲットドキュメントのスクリーン上の位置を、ピクセル単位で設定します。
     * @param documentTopOnScreen ドキュメントのスクリーン上の上端
     * @param documentLeftOnScreen ドキュメントの、スクリーン上の左端
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.updatePosition = function (documentTopOnScreen, documentLeftOnScreen) {

        this.documentTopOnScreen = documentTopOnScreen;
        this.documentLeftOnScreen = documentLeftOnScreen;
    };

    /**
     * イベントを発生させる間隔を、ミリ秒単位で設定します。
     * @pamar イベントを発生させる間隔
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.delay = function (ms) {

        this.applet.delay(ms);
    };

    /**
     * キーボードのキーを押下したイベントを発生させます。
     * 引数にkeycodeのみ指定した場合、モディファイキーに全てfalseを指定した場合と同じです。
     * @param keycode イベントを発生させるキーコード
     * @param isCtrlKey コントロールキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isShiftKey シフトキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isAltKey Altキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isMetaKey Metaキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.keyPress = function (keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        if ('undefined' == typeof isCtrlKey) {

            this.applet.keyPress(keycode);

        } else {

            this.applet.keyPress(keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey);
        }
    };

    /**
     * キーボードのキーを放したイベントを発生させます。
     * 引数にkeycodeのみ指定した場合、モディファイキーに全てfalseを指定した場合と同じです。
     * @param keycode イベントを発生させるキーコード
     * @param isCtrlKey コントロールキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isShiftKey シフトキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isAltKey Altキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isMetaKey Metaキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.keyRelease = function (keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        if ('undefined' == typeof isCtrlKey) {

            this.applet.keyRelease(keycode);

        } else {

            this.applet.keyRelease(keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey);
        }
    };

    /**
     * キーボードのキーを押して放したイベントを発生させます.
     * 引数にkeycodeのみ指定した場合、モディファイキーに全てfalseを指定した場合と同じです。
     * モディファイキーを指定した場合の、イベントの発生順は以下のとおりとなります。
     * <pre>
     * モディファイキーを押す
     *          ↓
     *  キーコードキーを押す
     *          ↓
     * モディファイキーを放す
     *          ↓
     *  キーコードキーを放す
     * </pre>
     * @param keycode イベントを発生させるキーコード
     * @param isCtrlKey コントロールキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isShiftKey シフトキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isAltKey Altキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isMetaKey Metaキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.keyType = function (keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        if (this.isAsyncKey) {

            Log.debug("---------- JSRobot#keyType Async ----------");

            var keyItem = new Object();
            keyItem.keycode = keycode;
            keyItem.isCtrlKey = isCtrlKey;
            keyItem.isShiftKey = isShiftKey;
            keyItem.isAltKey = isAltKey;
            keyItem.isMetaKey = isMetaKey;
            keyItem.robot = this;
            keyItem.action = function () {

                __keyType__(this.robot.applet,
                            this.keycode,
                            this.isCtrlKey,
                            this.isShiftKey,
                            this.isAltKey,
                            this.isMetaKey);
            };

            var jstimer = new com.yscjp.jsapp.tools.JSTimer(keyItem, 0);
            jstimer.start();

        } else {

            Log.debug("---------- JSRobot#keyType Sync ----------");

            __keyType__(this.applet, keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey);
        }

    };

    function __keyType__(applet, keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        if ('undefined' == typeof isCtrlKey) {

            applet.keyType(keycode);
//            applet.keyPress(keycode);
//            applet.keyRelease(keycode);

        } else {

            applet.keyPress(keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey);

            /*
             * リリース時にモディファイキーを押下した状態にすると、
             * 余分なイベントが発生して想定と異なる動きとなるので、
             * 押下しない。
             */
            applet.keyRelease(keycode);
            //this.applet.keyRelease(keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey);
        }
    }

    /**
     * キーボードで、指定のキャラクタを入力するイベントを発生させます.
     * 指定できるキャラクタは、表示可能なASCIIコードです。
     * シフトキーを同時に押下する必要があるキャラクタの場合、シフトキーの押下イベントを内部で発行します。
     * @param c キャラクタ
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.typeCharKey = function (c) {

        if (this.isAsyncKey) {

            Log.debug("---------- JSRobot#typeCharKey Async ----------");

            var keyItem = new Object();
            keyItem.charKey = c;
            keyItem.robot = this;
            keyItem.action = function () {

                this.robot.applet.typeCharKey(this.charKey);
            };

            var jstimer = new com.yscjp.jsapp.tools.JSTimer(keyItem, 0);
            jstimer.start();

        } else {

            Log.debug("---------- JSRobot#typeCharKey Sync ----------");

            this.applet.typeCharKey(c);
        }
    };

    /**
     * キーボードで、指定の文字列を入力するイベントを発生させます.
     * 指定できる文字列は、表示可能なASCIIコードです。
     * シフトキーを同時に押下する必要があるキャラクタの場合、シフトキーの押下イベントを内部で発行します。
     * @param c キャラクタ
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.inputText = function (text) {

        if (this.isAsyncKey) {

            Log.debug("---------- JSRobot#typeCharKey Async ----------");

            var keyItem = new Object();
            keyItem.text = text;
            keyItem.robot = this;
            keyItem.action = function () {

                this.robot.applet.inputText(this.text);
            };

            var jstimer = new com.yscjp.jsapp.tools.JSTimer(keyItem, 0);
            jstimer.start();

        } else {

            Log.debug("---------- JSRobot#typeCharKey Sync ----------");

            this.applet.inputText(text);
        }
    };

    /**
     * マウスカーソルを、指定の位置に移動させます.
     * 位置は、Webブラウザのクライアント領域の左上を0,0とする、ピクセルです。
     * 内部で、指定の値をスクリーン上の位置に変換して、マウスカーソルを移動させます。
     * @param {Number} x Webブラウザのクライアント領域の左上を0とする、X方向のピクセル
     * @param {Number} y Webブラウザのクライアント領域の左上を0とする、Y方向のピクセル
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMove = function (x, y) {

        if (!this.isOverScreen) {

            if (0 > x ||
                0 > y ||
                this.targetWindow.screen.width < x ||
                this.targetWindow.screen.height < y) {

                /*
                 * マウスドラッグ中の場合、マウスが効かなくなる危険性が有るので、
                 * 一旦リリースする。
                 */
                this.mouseLeftRelease();
                this.targetWindow.alert("マウスポインタを、Webブラウザの外に移動しようとしました。\n"+
                                        "処理を終了します。");
                throw new Error("Illegal position :" + x + "/" + y);
            }
        }

        //this.updatePosition();
//        Log.debug("com.yscjp.webtools.JSRobot#mouseMove x:" + x + " y:" + y);
        this.moveX = x;
        this.moveY = y;
        this.applet.mouseMove(this.documentLeftOnScreen + x,
                              this.documentTopOnScreen + y);
    };

    /**
     * マウスカーソルを、現地点から指定の移動量だけ移動させます.
     * 移動量は、ピクセルです。左方向または上方向への移動は、負数で指定します。
     * 内部で、指定の値をスクリーン上の位置に変換して、マウスカーソルを移動させます。
     * @param {Number} x X方向のピクセル。左方向への移動は、負数で指定します
     * @param {Number} y Y方向のピクセル。上方向への移動は、負数で指定します
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMoveFromHere = function (x, y) {

        this.mouseMove(this.moveX + x, this.moveY + y);
    };

    /**
     * マウスカーソルを、tagNameとindexで指定された要素の上に移動します.
     * indexは、以下のコードで返されるNodeListのindexです。
     * <pre>document.getElementsByTagName(tagName);</pre>
     * @param {String} tagName マウスを移動する要素タグ名称
     * @param {Number} index マウスを移動する要素インデックス
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMoveToTagNameAt = function (tagName, index, direction, x, y) {

        var targetElements = this.targetDocument.getElementsByTagName(tagName);
        if (index >= targetElements.length) {
            Log.error("Illegal argment. tagName:" + tagName + ' index:' + index);
            return;
        }
        this.mouseMoveTo(targetElements.item(index), direction, x, y);
    };

    /**
     * マウスカーソルを、idで指定された要素の上に移動します.
     * @param {String} id マウスを移動する要素のID
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMoveToId = function (id, direction, x, y) {

//        Log.debug("this.id:"+this.id);
        var targetElement = this.targetDocument.getElementById(id);
        if (!targetElement) {
            Log.error("Illegal id:" + id);
            return;
        }
        this.mouseMoveTo(targetElement, direction, x, y);
    };

    /**
     * マウスカーソルを、nameで指定された要素の上に移動します.
     * ドキュメント内に、同じnameの要素が複数存在する場合、先に見つけた要素に移動します。
     * @param {String} name マウスを移動する要素のname
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMoveToName = function (name, index, direction, x, y) {

        if ('undefined' == typeof index) {

            index = 0;
        }

        var targetElement = this.targetDocument.getElementsByName(name);
        if (!targetElement || index >= targetElement.length) {
            Log.error("Illegal name:" + name);
            return;
        }
        this.mouseMoveTo(targetElement.item(index), direction, x, y);
    };

    /**
     * マウスカーソルを、指定の要素の上に移動します.
     * @param {Element} targetElement マウスを移動する要素
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMoveTo = function(targetElement, direction, x, y) {

        if ('undefined' == typeof direction) {

            direction = "Center";
        }

        if ('undefined' == typeof x) {

            x = 0;
        }

        if ('undefined' == typeof y) {

            y = 0;
        }

        if (!targetElement) {
            Log.error("Argument is null.");
            return;
        }
        var htmlPosition = HtmlPosition.createHtmlPosition(targetElement);
        var point = htmlPosition.getPoint(direction);
//        Log.debug(point.toString());
        this.mouseMove(point.x + x, point.y + y);
    };

    /**
     * マウスのボタンを押すイベントを発生させます.
     * buttonsには、JSRobot.MOUSE_BUTTON1・JSRobot.MOUSE_BUTTON2・JSRobot.MOUSE_BUTTON3の、
     * 論理和を指定します。
     * @param buttons JSRobot.MOUSE_BUTTON1・JSRobot.MOUSE_BUTTON2・JSRobot.MOUSE_BUTTON3の論理和
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mousePress = function (buttons) {

        this.applet.mousePress(buttons);
    };

    /**
     * マウスのボタンを放すイベントを発生させます.
     * buttonsには、JSRobot.MOUSE_BUTTON1・JSRobot.MOUSE_BUTTON2・JSRobot.MOUSE_BUTTON3の、
     * 論理和を指定します。
     * @param buttons JSRobot.MOUSE_BUTTON1・JSRobot.MOUSE_BUTTON2・JSRobot.MOUSE_BUTTON3の論理和
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseRelease = function (buttons) {

        this.applet.mouseRelease(buttons);
    };

    /**
     * マウスの左ボタンを押すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseLeftPress = function () {

        this.applet.mousePress(true, false, false);
    };

    /**
     * マウスの左ボタンを放すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseLeftRelease = function () {

        this.applet.mouseRelease(true, false, false);
    };

    /**
     * マウスの中ボタンを押すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseCenterPress = function () {

        this.applet.mousePress(false, true, false);
    };

    /**
     * マウスの中ボタンを放すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseCenterRelease = function () {

        this.applet.mouseRelease(false, true, false);
    };

    /**
     * マウスの右ボタンを押すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseRightPress = function () {

        this.applet.mousePress(false, false, true);
    };

    /**
     * マウスの右ボタンを放すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseRightRelease = function () {

        this.applet.mouseRelease(false, false, true);
    };

    JSRobot.prototype.mouseWheel = function (wheelAmt) {

        this.applet.mouseWheel(wheelAmt);
    };

    /**
     * マウスの左ボタンをクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseLeftClick = function () {

        this.applet.mouseLeftClick();
    };

    /**
     * マウスの中ボタンをクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseCenterClick = function () {

        this.applet.mouseCenterClick();
    };

    /**
     * マウスの右ボタンをクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseRightClick = function () {

        this.applet.mouseRightClick();
    };

    /**
     * マウスの左ボタンをダブルクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseLeftDblclick = function () {

        this.applet.mouseLeftClick();
        this.applet.mouseLeftClick();
    };

    /**
     * マウスの中ボタンをダブルクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseCenterDblclick = function () {

        this.applet.mouseCenterClick();
        this.applet.mouseCenterClick();
    };

    /**
     * マウスの右ボタンをダブルクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseRightDblclick = function () {

        this.applet.mouseRightClick();
        this.applet.mouseRightClick();
    };

    /**
     * 指定のIDの要素をクリックして、フォーカスを移動させます。
     * @param id フォーカスを移動する要素のID
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.focusToId = function(id) {

        this.mouseMoveToId(id);
        this.mouseLeftClick();
    };

    /**
     * idで指定された要素にtextの内容を入力するイベントを発生させます。
     * このメソッドは、Firefoxでは、正しく動作しません。
     * @param id textの内容を入力する要素のID
     * @param text 入力する文字列
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.inputTextToId = function(id, text) {

        this.mouseMoveToId(id);
        this.mouseLeftClick();
        this.inputText(text);
    };

    JSRobot.prototype.windowCapture = function(title) {

        var captureWindow = this.targetWindow;

        if (!captureWindow) {
            captureWindow = window;
        }

        var x;
        var y;
        var height;
        var width;

        if ('undefined' == typeof captureWindow.screenX) {

            x = 0;
            y = 0;
            height = window.screen.height;
            width = window.screen.width;

        } else {

            x = captureWindow.screenX;
            y = captureWindow.screenY;
            height = captureWindow.outerHeight;
            width = captureWindow.outerWidth;
        }

        title = title == null ? "" : title;

        Log.debug("JSRobot#windowCapture x:" + x + " y:" + y + " width:" + width + " height:" + height);

        this.capture(x, y, width, height, title);
    };

    JSRobot.prototype.capture = function(x, y, width, height, title) {

        this.applet.capture(x, y, width, height, title);
    };
})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.webtools.JSRobot, "com.yscjp.webtools.JSRobot");
