/**
 * Web自動テストツール.
 * Webアプリケーションを、JavaのRobotクラスが生成するマウスやキーボードのイベントで、
 * 自動的にテストするツールです。
 * Java Appletが動作する環境が必要です。
 *
 * Version 00.10.00.02
 *
 * このプログラムのライセンスについて。
 * このプログラムは、MITライセンス(The MIT License)の元で公開します。
 * Released under the MIT Licenses.
 *
 * ライセンスの原文は、以下のURLを参照してください。
 * http://www.yscjp.com/licenses/MIT_LICENSE.txt
 *
 * Copyright (c) 2011-2012 Yagyu Teru, http://yscjp.com/
 *
 */

var com;
if (!com) com = {};
if (!com.yscjp) com.yscjp = {};
if (!com.yscjp.jsapp) com.yscjp.jsapp = {};
if (!com.yscjp.jsapp.webtest) com.yscjp.jsapp.webtest = {};

com.yscjp.jsapp.webtest.Webtest = function () {};
(function() {

    var Webtest = com.yscjp.jsapp.webtest.Webtest;

    var Log = com.yscjp.jsapp.util.Log;
    var JSEvent = com.yscjp.jsapp.JSEvent;
    var XmlUtil = com.yscjp.jsapp.util.XmlUtil;
    var ArgChk = com.yscjp.jsapp.util.ArgChk;

    var Autorun = com.yscjp.jsapp.autorun.Autorun;
    var EventCapture = com.yscjp.jsapp.autorun.EventCapture;

    var newWindow = null;
    var delayTime = 100;

    var captureNow = false;

    var jsRobot = null;

    /*
     * プライベートメソッドを管理する内部クラス。
     * トレースを取得する為に、一元管理していつ必要があるため、このような実装方法としている。
     */
    var __PFunc = function () {};

    function initializeWebtest(applet, isAsyncKey) {

        jsRobot = new com.yscjp.webtools.JSRobot(applet, isAsyncKey);
        jsRobot.id = "XXXXXXXXX";
        return jsRobot;
    }

    /**
     * テストケースを実行します.
     * @param {String} testcaseUrl 実行するテストシナリオのURL
     * @param {Number} _delayTime シナリオを実行するミリ秒単位の間隔。
     * @param {Number} _surveillanceinterval ドキュメントのロード判定をする間隔をミリ秒単位で指定します。
     * @param {Number} _maxSurveillanceCount ドキュメントのロードを待つ回数を指定します。
     * @memberOf com.yscjp.jsapp.webtest.Webtest
     */
    function execTestCase(testcaseUrl, _delayTime, _surveillanceinterval, _maxSurveillanceCount, _isOutStop) {

        delayTime = ArgChk.notUndefined(_delayTime, 100);

        captureNow = false;

        var isUseTargetwinTimer = document.getElementById("use_targetwin_timer").checked;

        var xmlDoc;
        var url = testcaseUrl + '?' + new Date().getTime();
        var xmlLS = new com.yscjp.jsapp.tools.XmlLS();
        xmlDoc = xmlLS.perseFromURI(url);
        if (null == xmlDoc) {

            var errorMsg = 'Http request error. ' +
            xmlLS.getResultCode() + ':' +
            xmlLS.getResultText() + '' +
            url;
            Log.error(errorMsg);
            throw errorMsg;
        }

        Autorun.initialize(xmlDoc,
                           delayTime,
                           __PFunc.loadListener,
                           __PFunc.unloadListener,
                           _surveillanceinterval,
                           _maxSurveillanceCount,
                           isUseTargetwinTimer,
                           _isOutStop);
        Autorun.run();
    }

    __PFunc.loadListener = function () {

        jsRobot.targetDocument = newWindow.document;
    };

    __PFunc.unloadListener = function () {
    };

    /**
     * コンテンツをロードします.
     * @param {String} loadURL ロードするコンテンツのURL
     * @param {String} openSetting オープンする window の設定内容
     * @memberOf com.yscjp.jsapp.webtest.Webtest
     */
    function loadContent(loadURL, openSetting) {

        newWindow = window.open(loadURL, "_blank", openSetting);
        newWindow.focus();
        isLoaded = false;
        Autorun.setNonSetTimeout();
        waitCount = 0;
        setTimeout("com.yscjp.jsapp.webtest.Webtest.contentLoaded()", 300);
    }

    function contentLoaded() {

        if ('complete'!= newWindow.document.readyState) {

            Log.debug('---------- contentLoaded not complete ----------');
            setTimeout('com.yscjp.jsapp.webtest.Webtest.contentLoaded()', 100);
            return;
        }

        Autorun.setTargetWindow(newWindow);
        JSEvent.addEventListener(newWindow.document, 'mousemove', __PFunc.mouseMoved, false);
        var windowTop = newWindow.screenY;
        var windowLeft = newWindow.screenX;
        if ("undefined" != typeof newWindow.screenY) {

            windowTop = newWindow.screenY;
            windowLeft = newWindow.screenX;

        } else if ("undefined" != typeof newWindow.screenTop) {

            windowTop = newWindow.screenTop;
            windowLeft = newWindow.screenLeft;

        } else {

            Log.debug('Window position unknown.');
            return;
        }

        var innerWidth = com.yscjp.jsapp.util.UA.getInnerWidth(newWindow);
        var innerHeight = com.yscjp.jsapp.util.UA.getInnerHeight(newWindow);

        Log.debug('windowTop:' + windowTop + ' windowLeft:' + windowLeft + ' innerHeight:' + innerHeight + ' innerWidth:' + innerWidth);
        jsRobot.setDocument(newWindow);
        jsRobot.mouseMove(windowLeft + (innerWidth / 2), windowTop + (innerHeight / 2));
        //jsRobot.mouseLeftClick();
    }

    __PFunc.mouseMoved = function (e) {

        e = JSEvent.createEvent(e);
        Log.debug('screenX:' + e.screenX +
                 ' screenY:' + e.screenY +
                 ' clientX:' + e.clientX +
                 ' clientY:' + e.clientY);
        jsRobot.updatePosition(e.screenY - e.clientY, e.screenX - e.clientX);
        isLoaded = true;
        JSEvent.removeEventListener(newWindow.document, 'mousemove', __PFunc.mouseMoved, false);

        if (captureNow) {

            EventCapture.setTargetWindow(newWindow);

            /*
             * イベントキャプチャ中
             * ここで直接captureInit()を呼ぶと、mouseClicked(e)が呼ばれた原因となった
             * イベントで、mouseclickListenerが呼ばれてしまうので、それをさけるために
             * 一定時間後に呼び出すようにする。
             */
            setTimeout('com.yscjp.jsapp.autorun.EventCapture.captureInit()', delayTime);

        } else {

            /*
             * テストケース実行中
             */
            setTimeout('com.yscjp.jsapp.autorun.Autorun.execute()', Autorun.delayTime);
        }
    };

    function capture(loadURL, openSetting, modifyEventCapture, pressReleaseCapture, labelclick) {

        captureNow = true;

        EventCapture.initialize(modifyEventCapture,
                                pressReleaseCapture,
                                labelclick);

        newWindow = window.open(loadURL, null, openSetting);
        newWindow.focus();
        EventCapture.setTargetWindow(newWindow);

        /*
         * イベントキャプチャ中
         * ここで直接captureInit()を呼ぶと、mouseClicked(e)が呼ばれた原因となった
         * イベントで、mouseclickListenerが呼ばれてしまうので、それをさけるために
         * 一定時間後に呼び出すようにする。
         */
        setTimeout('com.yscjp.jsapp.autorun.EventCapture.captureInit()', delayTime);
    }

    function manualTest(loadURL, openSetting, modifyEventCapture, pressReleaseCapture, labelclick) {

        captureNow = true;

        EventCapture.initialize(modifyEventCapture,
                                pressReleaseCapture,
                                labelclick,
                                __PFunc.captureOnloadHdl,
                                __PFunc.captureOnunloadHdl);

        newWindow = window.open(loadURL, null, openSetting);
        newWindow.focus();
        EventCapture.setTargetWindow(newWindow);

        /*
         * イベントキャプチャ中
         * ここで直接captureInit()を呼ぶと、mouseClicked(e)が呼ばれた原因となった
         * イベントで、mouseclickListenerが呼ばれてしまうので、それをさけるために
         * 一定時間後に呼び出すようにする。
         */
        setTimeout('com.yscjp.jsapp.autorun.EventCapture.manualTestInit()', delayTime);
    }

    function captureEnd() {

        captureNow = false;
        var capturetext = document.getElementById('capturetext');
        capturetext.value = EventCapture.captureEnd();

    }

    function captureExec(_delayTime, _surveillanceinterval, _maxSurveillanceCount, isOutStop) {

        delayTime = ArgChk.notUndefined(_delayTime, 100);

        var isUseTargetwinTimer = document.getElementById("use_targetwin_timer").checked;

        var xmlDoc = XmlUtil.perseFromString(document.getElementById('capturetext').value);

        Autorun.initialize(xmlDoc,
                           delayTime,
                           __PFunc.loadListener,
                           __PFunc.unloadListener,
                           _surveillanceinterval,
                           _maxSurveillanceCount,
                           isUseTargetwinTimer,
                           isOutStop);
        Autorun.run();
    }

    __PFunc.captureOnloadHdl = function () {

        //Webtest.imageCapture(newWindow.document.title + "_Onload");
        setTimeout("com.yscjp.jsapp.webtest.Webtest.imageCapture('" + newWindow.document.title + "_Onload')", 100);
    };

    __PFunc.captureOnunloadHdl = function () {

        Webtest.imageCapture(newWindow.document.title + "_Onunload");
    };

    com.yscjp.jsapp.util.StackTrace.insert(__PFunc, "com.yscjp.jsapp.webtest.Webtest.__PFunc");

    function imageCapture(title, top, left, height, width) {

        if ("undefined" == typeof top) {

            if (document.getElementById("imageCaptureSizeSpecify").checked) {

                top = document.getElementById("capture_top").value;
                left = document.getElementById("capture_left").value;
                height = document.getElementById("capture_height").value;
                width = document.getElementById("capture_width").value;

            } else {

                jsRobot.windowCapture(title);
                return;
            }
        }

        jsRobot.capture(left, top, width, height, title);
    }

    Webtest.initializeWebtest = initializeWebtest;
    Webtest.execTestCase = execTestCase;
    Webtest.loadContent = loadContent;
    Webtest.contentLoaded = contentLoaded;
    //Webtest.mouseMoved = mouseMoved;
    Webtest.capture = capture;
    Webtest.manualTest = manualTest;
    Webtest.captureEnd = captureEnd;
    Webtest.captureExec = captureExec;
    Webtest.imageCapture = imageCapture;

})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.webtest.Webtest,
                                       "com.yscjp.jsapp.webtest.Webtest");
