/**
 * JSTableのプラグイン
 *
 * Version 0.0.0.1
 *
 * このプログラムのライセンスについて。
 * このプログラムは、MITライセンス(The MIT License)の元で公開します。
 * Released under the MIT Licenses.
 *
 * ライセンスの原文は、以下のURLを参照してください。
 * http://www.yscjp.com/licenses/MIT_LICENSE.txt
 *
 * Copyright (c) 2010-2012 Yagyu Teru, http://yscjp.com/
 *
 */
//com.yscjp.jsapp.SelectablePlugin
//com.yscjp.jsapp.ChangeSizePlugin
//com.yscjp.jsapp.EditablePlugin
//com.yscjp.jsapp.PestePlugin

var com;
if (!com) com = {};
if (!com.yscjp) com.yscjp = {};
if (!com.yscjp.jsapp) com.yscjp.jsapp = {};

com.yscjp.jsapp.JSTablePlugin = function() {};
com.yscjp.jsapp.JSTablePlugin.init = function() {};
com.yscjp.jsapp.JSTablePlugin.fireSelectChanged = function() {};

/******************************************************************************
 * セルの選択を行うためのプラグイン
 *****************************************************************************/
com.yscjp.jsapp.SelectablePlugin = function (){};
com.yscjp.jsapp.SelectablePlugin.prototype = new com.yscjp.jsapp.JSTablePlugin();
com.yscjp.jsapp.SelectablePlugin.prototype.init = function (t) {

    /*
     * モジュールのインポート
     */
    var Log = com.yscjp.jsapp.util.Log;
    var JSEvent = com.yscjp.jsapp.JSEvent;
    var Table = com.yscjp.jsapp.html.Table;
    var JSHtmlUtil = com.yscjp.jsapp.util.JSHtmlUtil;

    var table = t;
    /* 処理対象のテーブル要素 */
    var tableElement = table.getTableElement();
    var htmlTable = new Table(tableElement);
    var id = tableElement.id;

    /*
     * テーブルに対するキーボードフォーカスを管理する。
     */
    /* 現在このテーブルにフォーカスがあるかのフラグ */
    var isTableSelected = false;
    /* このテーブルのフォーカスを管理する入力エリア */
    var tableCtrl = document.getElementById(id + "_ctrl");
    var headColumnCount = htmlTable.getHeadColumnCount();

    var selectedColumn = headColumnCount;
    var selectedRow = 0;

    var timerId;

    var tableTop = JSHtmlUtil.getTopPixel(tableElement);
    var tableLeft = JSHtmlUtil.getLeftPixel(tableElement);
    //table.fireSelectChanged(selectedRow, selectedColumn);
    //table.findCanFocusdCell(selectedRow, selectedColumn);
    selectedRow = table.getSelectedRowIndex();
    selectedColumn = table.getSelectedColumnIndex();
    table.fireSelectStart(selectedRow, selectedColumn);

    var selecting = false;

    /*
     * テーブル上で、マウスのボタンがdownされた時のイベントハンドラ
     */
    function selectStart (e) {

        e = JSEvent.createEvent(e);

        /*
         * 左ボタン以外は無視する
         */
        if (0 != e.button) {

            return;
        }
        //Log.debug("---------- selectStart ----------" + e.target.tagName);
        if ("TEXTAREA" == e.target.tagName) {
            return;
        }

        /*
         * イベントが発生した要素の先祖の要素に、TD要素が存在すること。
         */
        var element = getCellByPosition(e);
        if (isSelectableCell(element)) {

            selectedRow = element.parentNode.sectionRowIndex;
            selectedColumn = element.cellIndex;
            if (!table.isCellEditable(selectedRow, selectedColumn)) {

                return;
            }
            table.fireSelectChanged(selectedRow, selectedColumn);
            table.fireSelectStart(selectedRow, selectedColumn);
            selecting = true;
        }
    }
    JSEvent.addEventListener(tableElement, "mousedown", selectStart, false);

    function selectNow (e) {

        if (!selecting) {
            return;
        }

        e = JSEvent.createEvent(e);
        tableCtrl.focus();
        var element = getCellByPosition(e);
        if (!element) {
            return;
        }
        var rowIndex = element.parentNode.sectionRowIndex;
        var columnIndex = element.cellIndex;
        table.fireSelectEnd(rowIndex, columnIndex);
    }
    JSEvent.addEventListener(tableElement, "mousemove", selectNow, false);

    function selectOut (e) {

        if (!selecting) {
            return;
        }

        e = JSEvent.createEvent(e);

        var scrollTop = document.documentElement.scrollTop;
        if (0 == scrollTop) {
            scrollTop = document.body.scrollTop;
        }
        var scrollLeft = document.documentElement.scrollLeft;
        if (0 == scrollLeft) {
            scrollLeft = document.body.scrollLeft;
        }
        var tableTop = JSHtmlUtil.getTopPixel(tableElement) - scrollTop;
        var tableLeft = JSHtmlUtil.getLeftPixel(tableElement) - scrollLeft;

        /*
         * td要素から出る度にこのイベントが飛ぶので、
         * 本当にtableを出たか判定する。
         */
        if (tableLeft >= e.clientX ||
            tableLeft + tableElement.offsetWidth <= e.clientX ||
            tableTop >= e.clientY ||
            tableTop + tableElement.offsetHeight <= e.clientY) {

            tableCtrl.blur();
            tableCtrl.focus();
            selecting = false;
        }
    }
    JSEvent.addEventListener(tableElement, "mouseout", selectOut, true);

    /*
     * テーブル上で、マウスのボタンがupされた時のイベントハンドラ
     */
    function selectEnd (e) {

        if (!selecting) {

            tableCtrl.blur();
            tableCtrl.focus();
            return;
        }

        e = JSEvent.createEvent(e);
        tableCtrl.blur();
        tableCtrl.focus();
        var element = getCellByPosition(e);
        if (!isSelectableCell(element)) {
            return;
        }
        var rowIndex = element.parentNode.sectionRowIndex;
        var columnIndex = element.cellIndex;
        table.fireSelectEnd(rowIndex, columnIndex);
        selecting = false;
    }
    JSEvent.addEventListener(tableElement, "mouseup", selectEnd, false);

    function isSelectableCell(element) {

        if (null == element) {
            return false;
        }

        /*
         * 親要素がTHEADの場合は、選択不可。
         */
        if ("THEAD" == element.parentNode.parentNode.tagName) {

            return false;
        }

        /*
         * cellIndexがheadColumnCountより小さい場合は、選択不可。
         */
        if (element.cellIndex < headColumnCount) {

            return false;
        }

        return true;
    }

    /**
     * マウスイベントが発生した位置にあるセルを取得します。
     * マウスイベントが発生した位置にセルが存在しない場合、nullを返します。
     */
    function getCellByPosition(e) {

        var positionX = e.clientX;
        var positionY = e.clientY;

        /*
         * 行を特定する。
         */
        var rowElement = null;
        var scrollTop = document.documentElement.scrollTop;
        if (0 == scrollTop) {
            scrollTop = document.body.scrollTop;
        }
        for (var i = 0; i < htmlTable.getBodiesRowCount(); i ++) {

            var tmpRow = htmlTable.getRowFromBodiesAt(i);
            //var rowTop = tmpRow.offsetTop + tableTop;
            //var rowBottom = rowTop + tmpRow.offsetHeight;
            var rowTop = JSHtmlUtil.getTopPixel(tmpRow) - scrollTop;
            var rowBottom = rowTop + tmpRow.offsetHeight;
        //Log.debug("---------- getCellByPosition ---------- [" + i + "]" +
        //        "positionY:"+positionY+" rowTop:"+rowTop+" rowBottom:"+rowBottom);
            if (rowTop <= positionY && positionY <= rowBottom) {

                rowElement = tmpRow;
                break;
            }
        }

        if (null == rowElement) {

            return null;
        }

        /*
         * セルを特定する。
         */
        var cells = rowElement.cells;
        for (var i = 0; i < cells.length; i ++) {

            var cell = cells.item(i);
            var cellLeft = cell.offsetLeft + tableLeft;
            var cellRight = cellLeft + cell.offsetWidth;
            if (cellLeft <= positionX && positionX <= cellRight) {

                return cell;
            }
        }

        return null;
    }

    /*
     * キーボードによりセルの選択が変更された時のイベントハンドラ
     */
    function cellSelect(e) {

        e = JSEvent.createEvent(e);

        //e.preventDefault();
        //e.stopPropagation();


        //Log.debug("---------- keydown ----------");
        //Log.debug("e.keyIdentifier(" + e.keyIdentifier + ")");
        //Log.debug("sectionPaperWraper.className:" + sectionPaperWraper.className);

        var isMove = false;

        switch (e.keyIdentifier) {
            case 37 : // 左
                isMove = table.goLeft();
                break;
            case 38 : // 上
                isMove = table.goUp();
                break;
            case 39 : // 右
                isMove = table.goRight();
                break;
            case 40 : // 下
                isMove = table.goDown();
                break;
            case 13 : // Return
                break;
            default :
                return;
        }

        selectedColumn = table.getSelectedColumnIndex();
        selectedRow = table.getSelectedRowIndex();
        if (e.shiftKey) {

            /*
             * Shiftキーが押下されている
             */
            table.fireSelectEnd(selectedRow, selectedColumn);

        } else {

            table.fireSelectStart(selectedRow, selectedColumn);

            if (isMove) {

                keydownEvent = e.nativeEvent;
                if (null != timerId) {
                    clearTimeout(timerId);
                    timerId = null;
                }
            }
        }
    }
    JSEvent.addEventListener(tableCtrl, "keydown", cellSelect, false);

    /**
     * keydownイベントを繰り返す処理。
     * 最初は200ミリ秒、それ以降は100ミリ秒でイベントを繰り返す。
     */
    var repeatTime = 200;
    var keydownEvent;
    function repeatKeydown() {
        repeatTime = 100;
        cellSelect(keydownEvent);
    }

    /**
     * keyupで繰り返しをキャンセルする。
     * イベント間隔を、最初の200ミリ秒にリセットする。
     */
//    function stopKeydown() {

//        repeatTime = 200
//        clearTimeout(timerId);
//        timerId = null;
//    }
//    JSEvent.addEventListener(tableCtrl, "keyup", stopKeydown, false);

    /*
     * フォーカスを受け取った時点で呼び出されるイベントハンドラ
     */
    function tableOnFocus(e) {

        //Log.debug("---------- tableOnFocus ----------");
        e = JSEvent.createEvent(e);
        JSHtmlUtil.addClassValue(tableElement, "com_yscjp_htmlclass_selected");
        isTableSelected = true;
    }
    JSEvent.addEventListener(tableCtrl, "focus", tableOnFocus, false);

    /*
     * フォーカスを失った時点で呼び出されるイベントハンドラ
     */
    function tableOnBlur(e) {
        //Log.debug("---------- tableOnBlur ----------");
        e = JSEvent.createEvent(e);
        JSHtmlUtil.removeClassValue(tableElement, "com_yscjp_htmlclass_selected");
        isTableSelected = false;
    }
    JSEvent.addEventListener(tableCtrl, "blur", tableOnBlur, false);

};

/******************************************************************************
 * セルの幅及び高さを変更できるプラグイン
 *****************************************************************************/
com.yscjp.jsapp.ChangeSizePlugin = function (){};
com.yscjp.jsapp.ChangeSizePlugin.prototype = new com.yscjp.jsapp.JSTablePlugin();
com.yscjp.jsapp.ChangeSizePlugin.prototype.init = function (t) {

    /*
     * モジュールのインポート
     */
    var Log = com.yscjp.jsapp.util.Log;
    var JSEvent = com.yscjp.jsapp.JSEvent;
    var Table = com.yscjp.jsapp.html.Table;
    var JSHtmlUtil = com.yscjp.jsapp.util.JSHtmlUtil;
    var DomUtil = com.yscjp.jsapp.util.DomUtil;

    var table = t;
    /* 処理対象のテーブル要素 */
    var tableElement = table.getTableElement();
    var htmlTable = new Table(tableElement);
    var id = tableElement.id;
    var headColumnCount = htmlTable.getHeadColumnCount();

    var tableTop = tableElement.offsetTop;
    var tableLeft = tableElement.offsetLeft;

    /* CELLの高さや幅を、変更できるようにする。 */
    /* 列ヘッダのCELLに、イベントリスナを登録する */
    var nodeList;

    for (var headRowIndex = 0; headRowIndex < htmlTable.getHeadRowCount(); headRowIndex ++) {
        nodeList = htmlTable.getRowFromHeadAt(headRowIndex).cells;
        for (var i = headColumnCount; i < nodeList.length; i++) {

            var headCell = nodeList.item(i);

            JSEvent.addEventListener(headCell, "mousemove", mousemoveColumnHead, false);
            JSEvent.addEventListener(headCell, "mousedown", hselected, false);
        }
    }

    var rowCount = htmlTable.getBodiesRowCount();
    for (var rowIndex = 0; rowIndex < rowCount; rowIndex ++) {

        for (var columnIndex = 0; columnIndex < headColumnCount; columnIndex ++) {

            var headCell = htmlTable.getCellFromBodiesAt(rowIndex, columnIndex);

            JSEvent.addEventListener(headCell, "mousemove", mousemoveRowHead, false);
            JSEvent.addEventListener(headCell, "mousedown", vselected, false);
        }
    }

    function mousemoveColumnHead(e) {

        //Log.debug("---------- mousemoveColumnHead ----------");
        e = JSEvent.createEvent(e);

        if (isRightEnd(e)) {

            e.target.style.cursor = "col-resize";

        } else {

            e.target.style.cursor = "";
        }
    }

    function mousemoveRowHead(e) {

        //Log.debug("---------- mousemoveColumnHead ----------");
        e = JSEvent.createEvent(e);

        if (isBottomEnd(e)) {

            e.target.style.cursor = "row-resize";

        } else {

            e.target.style.cursor = "";
        }
    }

    function release(e) {

        e = JSEvent.createEvent(e);

        selectedElement = null;
        cursorPositionX = -1;
        cursorPositionY = -1;

        JSEvent.removeEventListener(document, "mouseup", release, false);
        JSEvent.removeEventListener(document, "mousemove", hmoving, false);
        JSEvent.removeEventListener(document, "mousemove", vmoving, false);

        table.focus();
    };

    function hmoving(e) {

        var PADDIND_WIDTH = 2;

        e = JSEvent.createEvent(e);

        /*
         * セルのスタイルシート上の幅を、ピクセル単位に取得する。
         * 要素のclientWidthに、パディングのサイズをプラスして取得する。
         * TODO スタイルシートで指定されたパディングサイズを、
         * ピクセル単位で取得する方法が、今のところ不明。
         */
        var styleWidth = selectedElement.clientWidth - PADDIND_WIDTH;

        var newWidth = styleWidth + (e.clientX - cursorPositionX);
        selectedElement.style.width = newWidth + "px";
        cursorPositionX = e.clientX;
        table.over();
    };

    function vmoving(e) {

        var PADDING_HEIGHT = 0;

        e = JSEvent.createEvent(e);

        /*
         * セルのスタイルシート上の幅を、ピクセル単位に取得する。
         * 要素のclientWidthに、パディングのサイズをプラスして取得する。
         * TODO スタイルシートで指定されたパディングサイズを、
         * ピクセル単位で取得する方法が、今のところ不明。
         */
        var styleHeight = selectedElement.style.height;
        if (styleHeight) {

            styleHeight = styleHeight.replace("px", "") - 0;

        } else {

            styleHeight = selectedElement.clientHeight - PADDING_HEIGHT;
        }

        var newHeight = styleHeight + (e.clientY - cursorPositionY);
        selectedElement.style.height = newHeight + "px";
        cursorPositionY = e.clientY;
        table.over();
    };

    function hselected(e) {

        e = JSEvent.createEvent(e);

        if (!isRightEnd(e)) {

            return;
        }

        selectedElement = e.currentTarget;
        cursorPositionX = e.clientX;
        cursorPositionY = e.clientY;

        JSEvent.addEventListener(document, "mouseup", release, false);
        JSEvent.addEventListener(document, "mousemove", hmoving, false);
    };

    function vselected(e) {

        e = JSEvent.createEvent(e);

        if (!isBottomEnd(e)) {

            return;
        }

        selectedElement = e.currentTarget;
        cursorPositionX = e.clientX;
        cursorPositionY = e.clientY;

        JSEvent.addEventListener(document, "mouseup", release, false);
        JSEvent.addEventListener(document, "mousemove", vmoving, false);
    };

    function isRightEnd(e) {
        var rightEnd = tableLeft + e.target.offsetLeft + e.target.offsetWidth;
        var rightRange = rightEnd - 10;
        var positionX = e.clientX;
        if (rightRange <= positionX && positionX <= rightEnd) {
            return true;
        } else {
            return false;
        }
    };

    function isBottomEnd(e) {
        var bottomEnd = tableTop + e.target.offsetTop + e.target.offsetHeight;
        var bottomRange = bottomEnd - 10;
        var positionY = e.clientY;
        if (bottomRange <= positionY && positionY <= bottomEnd) {
            return true;
        } else {
            return false;
        }
    };

    table.over();
};


/******************************************************************************
 * セルの内容を変更できるプラグイン
 *****************************************************************************/
com.yscjp.jsapp.EditablePlugin = function (){};
com.yscjp.jsapp.EditablePlugin.prototype = new com.yscjp.jsapp.JSTablePlugin();
com.yscjp.jsapp.EditablePlugin.prototype.init = function (t) {

    /*
     * モジュールのインポート
     */
    var Log = com.yscjp.jsapp.util.Log;
    var JSEvent = com.yscjp.jsapp.JSEvent;
    var JSHtmlUtil = com.yscjp.jsapp.util.JSHtmlUtil;
    var DomUtil = com.yscjp.jsapp.util.DomUtil;

    var table = t;
    /* 処理対象のテーブル要素 */
    var tableElement = table.getTableElement();
    var id = tableElement.id;
    var htmlTable = table.htmlTable;
    /* このテーブルのフォーカスを管理する入力エリア */
    var tableCtrl = document.getElementById(id + "_ctrl");
    //var cellOver = document.getElementById(id + "_cellOver");

    var textarea = null;
    var timerId;
    var textareaMinHeight = 0;

    var getValueRowIndex;
    var getValueColumnIndex;

    var editareaTopCell;
    var editareaBottomCell;

    /**
     * マウスにより編集エリアを開くイベント
     */
    function mouseEditable(e) {

        if (null != textarea) {
            return;
        }

        setTimeout(openEdtior, 0);
    }
    //JSEvent.addEventListener(cellOver, "dblclick", mouseEditable, false);
    JSEvent.addEventListener(tableElement, "dblclick", mouseEditable, false);

    /**
     * 編集エリアを開くイベント
     */
    function editable(e) {

        //Log.debug("---------- editable ----------");
        e = JSEvent.createEvent(e);

        switch (e.keyIdentifier) {
            case 113 : // F2
                break;
            case 13 : // Enter
                break;
            case 46 : // Delete
                clearCells();
                return;
            default :
                return;
        }
        setTimeout(openEdtior, 0);
    }
    JSEvent.addEventListener(tableCtrl, "keydown", editable, false);

    function clearCells() {

        var pasteRow = table.selectedTopRow;
        var pasteColumn = table.selectedLeftColumn;
        var bottomRow = table.selectedBottomRow;
        var rightColumn = table.selectedRightColumn;

        for (var rowIndex = pasteRow; rowIndex < bottomRow + 1; rowIndex ++) {

            for (var columnIndex = pasteColumn; columnIndex < rightColumn + 1; columnIndex ++) {

                table.setValueAt("", rowIndex, columnIndex);
            }
        }
    }

    function openEdtior() {

        var row = table.getSelectedRowIndex();
        var col = table.getSelectedColumnIndex();
        if (! table.isCellEditable(row, col)) {
            return;
        }

        //Log.debug("---------- openEdtior ----------");
        textarea = document.createElement("TEXTAREA");
        //textarea = cellOver.appendChild(textarea);
        textarea = tableElement.parentNode.insertBefore(textarea, tableElement);
        JSEvent.addEventListener(textarea, "focus", editorOnFocus, false);
        JSEvent.addEventListener(textarea, "blur", editorOnBlur, false);
        JSEvent.addEventListener(textarea, "keydown", editEnd, false);
        textarea.style.display = "block";
        textarea.style.position = "absolute";
        editareaTopCell = table.currentCell;
        editareaBottomCell = table.currentCell;
        textarea.style.top = JSHtmlUtil.getTopPixel(table.currentCell) + 'px';
        textarea.style.left = JSHtmlUtil.getLeftPixel(table.currentCell) + 'px';
        textarea.style.height = JSHtmlUtil.getHeightPixel(table.currentCell) + 'px';
        textarea.style.width = JSHtmlUtil.getWidthPixel(table.currentCell) + 'px';
        //textarea.style.height = "100%";
        //textarea.style.width = "100%";
        textarea.style.border = "none";
        textarea.style.overflow = "hidden";
        textarea.style.padding = "0";
        textarea.style.backgroundColor = "#ffffff";
        textarea.style.zIndex = '3';
        textarea.focus();
        //Log.debug("textarea.clientHeight" + textarea.clientHeight + " textarea.clientWidth" + textarea.clientWidth);
        JSEvent.addEventListener(textarea, "keyup", keyUpHandorer, false);
        JSEvent.addEventListener(textarea, "keyuo", keyUpHandorer, false);
//        timerId = setTimeout(keyUpHandorer, 100);
        var value = table.getValueCurrentCell();
        getValueRowIndex = table.getSelectedRowIndex();
        getValueColumnIndex = table.getSelectedColumnIndex();// + table.headColumnCount;
        textarea.value = value;
        textarea.select();
        textareaMinHeight = textarea.clientHeight;
    }

    /*
     * フォーカスを受け取った時点で呼び出されるイベントハンドラ
     */
    function editorOnFocus(e) {
        //Log.debug("---------- editorOnFocus ----------");
        e = JSEvent.createEvent(e);
        JSHtmlUtil.addClassValue(tableElement, "com_yscjp_htmlclass_selected");
    }

    /*
     * フォーカスを失った時点で呼び出されるイベントハンドラ
     */
    function editorOnBlur(e) {
        //Log.debug("---------- editorOnBlur ----------");
        e = JSEvent.createEvent(e);
        closeEdtior();
        JSHtmlUtil.removeClassValue(tableElement, "com_yscjp_htmlclass_selected");
    }

    /*
     * 入力によってテキストエリアのサイズを変更するイベントハンドラ。
     * テキストエリアの表示サイズより、スクロールサイズが大きくなった場合、
     * 表示サイズを大きくする。
     */
    function keyUpHandorer(e) {

        if (textarea.offsetHeight < textarea.scrollHeight) {

            var nextRowHeight = 0;
            var tmpEditareaBottomCell = htmlTable.getBelowCellFromSection(editareaBottomCell);
            if (null != tmpEditareaBottomCell) {

                /*
                 * カレント行が最下行の場合、nullが返る。
                 * その場合は、追加する高さは0のまま。
                 */
                nextRowHeight = JSHtmlUtil.getHeightPixel(tmpEditareaBottomCell);
                editareaBottomCell = tmpEditareaBottomCell;
            }
            textarea.style.height = textarea.offsetHeight + nextRowHeight + "px";
        }
    }

    /**
     * 編集エリアを閉じるイベント
     */
    function editEnd(e) {

        //Log.debug("---------- editEnd ----------");
        e = JSEvent.createEvent(e);

        switch (e.keyIdentifier) {
            case 13 : // Enter
                break;
            default :
                return;
        }
        closeEdtior();
    }

    function closeEdtior() {

        JSEvent.removeEventListener(textarea, "focus", editorOnFocus, false);
        JSEvent.removeEventListener(textarea, "blur", editorOnBlur, false);
        JSEvent.removeEventListener(textarea, "keydown", editEnd, false);
        JSEvent.removeEventListener(textarea, "keyup", keyUpHandorer, false);
        JSEvent.removeEventListener(textarea, "keyuo", keyUpHandorer, false);

        table.setValueAt(textarea.value, getValueRowIndex, getValueColumnIndex);
        //DomUtil.removeAllChild(cellOver);
        tableElement.parentNode.removeChild(textarea);
        clearTimeout(timerId);
        textarea = null;
        table.over();
        setTimeout(focusTable, 0);
    }

    function focusTable() {
        tableCtrl.focus();
    }
};


/******************************************************************************
 * セルの内容をコピー＆ペーストできるプラグイン
 *****************************************************************************/
com.yscjp.jsapp.PestePlugin = function (){};
com.yscjp.jsapp.PestePlugin.prototype = new com.yscjp.jsapp.JSTablePlugin();
com.yscjp.jsapp.PestePlugin.prototype.init = function (t) {

    /*
     * モジュールのインポート
     */
    var Log = com.yscjp.jsapp.util.Log;
    var JSEvent = com.yscjp.jsapp.JSEvent;
    var JSHtmlUtil = com.yscjp.jsapp.util.JSHtmlUtil;
    var DomUtil = com.yscjp.jsapp.util.DomUtil;

    var table = t;
    /* 処理対象のテーブル要素 */
    var tableElement = table.getTableElement();
    var id = tableElement.id;
    /* このテーブルのフォーカスを管理する入力エリア */
    var tableCtrl = document.getElementById(id + "_ctrl");
    //var cellOver = document.getElementById(id + "_cellOver");
    var pasteValues;

    var textarea;
    var timerId;

    function tableCtrlSelectHdl(e) {

        setTimeout(tableCtrlSelect, 0);
    }

    function tableCtrlSelect() {

        tableCtrl.value = " ";
        tableCtrl.select();
    }
    tableCtrlSelect();

    JSEvent.addEventListener(tableCtrl, "keyup", tableCtrlSelectHdl, false);
    JSEvent.addEventListener(tableCtrl, "focus", tableCtrlSelectHdl, false);

    /**
     * コピーボタン押下イベント
     */
    function copyBefor(e) {

        Log.debug("---- copyBefor ----");

        e = JSEvent.createEvent(e);

        tableCtrl.value = table.getValueCurrentCell();
        tableCtrl.select();

//        setTimeout(copyAfter, 0);
    }

    JSEvent.addEventListener(tableCtrl, "copy", copyBefor, false);

//    function copyAfter() {
//    }

    var pasteRow;
    var pasteColumn;
    var pastedRow;
    var pastedColumn;
    var columnPasteCount = 0;
    var rowPasteCount = 0;

    /**
     * ペーストボタン押下イベント
     */
    function pasteBefor(e) {

        Log.debug("---- pasteBefor ----");

        e = JSEvent.createEvent(e);

        tableCtrl.value = "";
        pasteRow = table.selectedTopRow;
        pasteColumn = table.selectedLeftColumn;
        pastedRow = -1;
        pastedColumn = -1;

        setTimeout(pasteAfter, 0);
    }

    JSEvent.addEventListener(tableCtrl, "paste", pasteBefor, false);

    /**
     * ペーストイベントのハンドラから、非同期に呼び出されます。
     * ペースト処理を行います。
     */
    function pasteAfter() {

        createPasteValues();

        var bottomRow = table.selectedBottomRow;
        var rightColumn = table.selectedRightColumn;

        for (var rowIndex = pasteRow; rowIndex < bottomRow + 1; rowIndex += rowPasteCount) {

            for (var columnIndex = pasteColumn; columnIndex < rightColumn + 1; columnIndex += columnPasteCount) {

                paste(rowIndex, columnIndex);
            }
        }
    }

    /**
     * ペーストされた文字列を、'\n'と'\t'で分割し、2次元配列に格納します。
     */
    function createPasteValues() {

        columnPasteCount = 0;
        rowPasteCount = 0;
        var pasteValue = tableCtrl.value;

        /*
         * 文字列の最後が改行の場合、改行を削除する。
         */
        if ("\n" == pasteValue.charAt(pasteValue.length - 1)) {

            pasteValue = pasteValue.substring(0, pasteValue.length - 1);
        }

        var lineSplitValues = pasteValue.split("\n");
        rowPasteCount = lineSplitValues.length;
        pasteValues = new Array(rowPasteCount);
        for (var rowIndex = 0; rowIndex < lineSplitValues.length; rowIndex ++) {

            var row = lineSplitValues[rowIndex];
            var tabSplitValues = row.split("\t");
            pasteValues[rowIndex] = new Array(tabSplitValues.length);
            columnPasteCount = Math.max(tabSplitValues.length, columnPasteCount);
            for (var columnIndex = 0; columnIndex < tabSplitValues.length; columnIndex ++) {

                pasteValues[rowIndex][columnIndex] = tabSplitValues[columnIndex];
            }
        }
    }

    /**
     * ペーストされた内容から作成された配列を、指定のインデックスを開始位置としてペーストします。
     */
    function paste(startRowIndex, startColumnIndex) {

        for (var rowIndex = 0; rowIndex < pasteValues.length; rowIndex ++) {

            var pasteRowIndex = startRowIndex + rowIndex;

            var pasteValuesRow = pasteValues[rowIndex];
            for (var columnIndex = 0; columnIndex < pasteValuesRow.length; columnIndex ++) {

                var pasteColumnIndex = startColumnIndex + columnIndex;

                if (!table.isIndexValid(pasteRowIndex, pasteColumnIndex)) {

                    continue;
                }

                table.setValueAt(pasteValues[rowIndex][columnIndex], pasteRowIndex, pasteColumnIndex);
            }
        }
    }
};

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.SelectablePlugin, "com.yscjp.jsapp.SelectablePlugin");
com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.ChangeSizePlugin, "com.yscjp.jsapp.ChangeSizePlugin");
com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.EditablePlugin, "com.yscjp.jsapp.EditablePlugin");
com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.PestePlugin, "com.yscjp.jsapp.PestePlugin");
