/**
 * Webアプリケーション自動テストユーティリティ。
 * Webアプリケーションを自動的にテストするための、ユーティリティです。
 * このライブラリは、javaアプレットを使用します。
 *
 * Version 00.12.00.00
 *
 * このプログラムのライセンスについて。
 * このプログラムは、MITライセンス(The MIT License)の元で公開します。
 * Released under the MIT Licenses.
 *
 * ライセンスの原文は、以下のURLを参照してください。
 * http://www.yscjp.com/licenses/MIT_LICENSE.txt
 *
 * Copyright (c) 2010-2013 Yagyu Teru, http://yscjp.com/
 *
 */
var com;
if (!com) com = {};
if (!com.yscjp) com.yscjp = {};
if (!com.yscjp.webtools) com.yscjp.webtools = {};

/**
 * serverPortで初期化したJSRobotクラスのインスタンスを作成します。
 * @class Javaのjava.awt.Robotを使用して、ネイティブのイベントを発生させます。
 */
com.yscjp.webtools.JSRobot = function(serverPortFunc) {

	this.serverPortFunc = serverPortFunc;
    this.isOverScreen = false;
    this.delayedFire = 0;
};

(function() {

    var HtmlPosition = com.yscjp.jsapp.tools.HtmlPosition;
    var Log = com.yscjp.jsapp.util.Log;

    /**
     * JSRobotクラス
     */
    var JSRobot = com.yscjp.webtools.JSRobot;
    JSRobot.exitHandler;
    JSRobot.errorHandler;

    JSRobot.prototype.send = function (command) {

        var serverPort = this.serverPortFunc();
        if (isNaN(serverPort)) {
            alert("サーバポートが数値ではありません。" + serverPort);
            return;
        }
        var serverUrl = "http://localhost:" + serverPort + "/";
        Log.debug('serverUrl;' + serverUrl);

        if (this.delayedFire && !isNaN(this.delayedFire)) {

            command = "DelayedFire," + this.delayedFire + "," + command;
        }

        this.delayedFire = 0;

        com.yscjp.jsapp.util.Request.post(serverUrl,
                                          "text/plain",
                                          command,
        function () {

            if (JSRobot.exitHandler) {

                try {

                    JSRobot.exitHandler();

                } finally {

                    JSRobot.exitHandler = null;
                    JSRobot.errorHandler = null;
                }
            }

        },
        function (error) {

            if (this.targetWindow && !this.targetWindow.closed) {

                this.targetWindow.alert("サーバへのリクエストに失敗しました。[" + error + "] " + command);

            } else {

                alert("サーバへのリクエストに失敗しました。[" + error + "] " + command);
            }

            if (JSRobot.errorHandler) {

                try {

                    JSRobot.errorHandler();

                } finally {

                    JSRobot.exitHandler = null;
                    JSRobot.errorHandler = null;
                }
            }
        });
    };

    /**
     * テストを行うターゲットのWindowを設定します。
     * @param targetWindow ターゲットのWindowオブジェクト
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.setDocument = function (targetWindow) {

        this.targetWindow = targetWindow;
        if (null != this.targetWindow) {

            this.targetDocument = this.targetWindow.document;

        } else {

            this.targetDocument = document;
        }

        this.documentTopOnScreen = 0;
        this.documentLeftOnScreen = 0;
    };
    //this.setDocument(targetWindow);

    /**
     * テストを行うターゲットドキュメントのスクリーン上の位置を、ピクセル単位で設定します。
     * @param documentTopOnScreen ドキュメントのスクリーン上の上端
     * @param documentLeftOnScreen ドキュメントの、スクリーン上の左端
     * @returns boolean
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.updatePosition = function (documentTopOnScreen, documentLeftOnScreen) {

        if ("undefined" == typeof documentTopOnScreen) {

            /*
             * 引数なし
             */
            if ("undefined" != typeof window.mozInnerScreenX) {

                /*
                 * Firefox
                 */
                this.documentTopOnScreen = this.targetWindow.mozInnerScreenY;
                this.documentLeftOnScreen = this.targetWindow.mozInnerScreenX;

            } else if ("undefined" != typeof window.screenLeft) {

                if ("undefined" != typeof window.screenX) {

                    if (window.screenLeft == window.screenX && window.screenTop == window.screenY) {

                        /*
                         * WebKit
                         */
                        //alert("お使いのWebブラウザでは、引数なしでこのメソッドを呼ぶことはできません。");
                        return false;
                    }
                }

                /*
                 * IE or Opera
                 */
                this.documentTopOnScreen = this.targetWindow.screenTop;
                this.documentLeftOnScreen = this.targetWindow.screenLeft;
            }
        } else {

            this.documentTopOnScreen = documentTopOnScreen;
            this.documentLeftOnScreen = documentLeftOnScreen;
        }

        Log.debug("this.documentTopOnScreen:" + this.documentTopOnScreen + " this.documentLeftOnScreen:" + this.documentLeftOnScreen);

        return true;
    };

    /**
     * ターゲットドキュメントのスクリーン上の上端位置を、ピクセル単位で返します。
     * @returns ターゲットドキュメントのスクリーン上の上端位置
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.getDocumentTopOnScreen = function () {

        this.updatePosition();
        return this.documentTopOnScreen;
    };

    /**
     * ターゲットドキュメントのスクリーン上の左端位置を、ピクセル単位で返します。
     * @returns ターゲットドキュメントのスクリーン上の上端位置
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.getDocumentLeftOnScreen = function () {

        this.updatePosition();
        return this.documentLeftOnScreen;
    };

    /**
     * イベントを発生させる間隔を、ミリ秒単位で設定します。
     * @param イベントを発生させる間隔
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.delay = function (ms) {

        this.applet.delay(ms);
    };

    /**
     * キーボードのキーを押下したイベントを発生させます。
     * 引数にkeycodeのみ指定した場合、モディファイキーに全てfalseを指定した場合と同じです。
     * @param keycode イベントを発生させるキーコード
     * @param isCtrlKey コントロールキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isShiftKey シフトキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isAltKey Altキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isMetaKey Metaキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.keyPress = function (keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.sendKeyCommand("KeyPress", keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey);
    };

    /**
     * キーボードのキーを放したイベントを発生させます。
     * 引数にkeycodeのみ指定した場合、モディファイキーに全てfalseを指定した場合と同じです。
     * @param keycode イベントを発生させるキーコード
     * @param isCtrlKey コントロールキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isShiftKey シフトキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isAltKey Altキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isMetaKey Metaキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.keyRelease = function (keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.sendKeyCommand("KeyRelease", keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey);
    };

    JSRobot.prototype.sendKeyCommand = function (command, keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        var commandText = command + "," + keycode + ",";
        commandText += getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey);
        this.send(commandText);
    };

    function getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        var modifyText = "";
        modifyText += isCtrlKey ? "TRUE," : "FALSE,";
        modifyText += isShiftKey ? "TRUE," : "FALSE,";
        modifyText += isAltKey ? "TRUE," : "FALSE,";
        modifyText += isMetaKey ? "TRUE," : "FALSE";

        return modifyText;
    }

    /**
     * キーボードのキーを押して放したイベントを発生させます.
     * 引数にkeycodeのみ指定した場合、モディファイキーに全てfalseを指定した場合と同じです。
     * モディファイキーを指定した場合の、イベントの発生順は以下のとおりとなります。
     * <pre>
     * モディファイキーを押す
     *          ↓
     *  キーコードキーを押す
     *          ↓
     * モディファイキーを放す
     *          ↓
     *  キーコードキーを放す
     * </pre>
     * @param keycode イベントを発生させるキーコード
     * @param isCtrlKey コントロールキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isShiftKey シフトキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isAltKey Altキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @param isMetaKey Metaキーを押している状態のイベントを発生させる場合true、
     * それ以外の場合falseを設定する。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.keyType = function (keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.sendKeyCommand("KeyType", keycode, isCtrlKey, isShiftKey, isAltKey, isMetaKey);
    };

    /**
     * キーボードで、指定の文字列を入力するイベントを発生させます.
     * 指定できる文字列は、表示可能なASCIIコードです。
     * シフトキーを同時に押下する必要があるキャラクタの場合、シフトキーの押下イベントを内部で発行します。
     * @param c キャラクタ
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.inputText = function (text) {

        this.send("TypeText," + text);
    };

    /**
     * マウスカーソルを、指定の位置に移動させます.
     * 位置は、Webブラウザのクライアント領域の左上を0,0とする、ピクセルです。
     * 内部で、指定の値をスクリーン上の位置に変換して、マウスカーソルを移動させます。
     * @param {Number} x Webブラウザのクライアント領域の左上を0とする、X方向のピクセル
     * @param {Number} y Webブラウザのクライアント領域の左上を0とする、Y方向のピクセル
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMove = function (x, y) {

        if (!this.isOverScreen) {

            if (0 > x ||
                0 > y ||
                this.targetWindow.screen.width < x ||
                this.targetWindow.screen.height < y) {

                /*
                 * マウスドラッグ中の場合、マウスが効かなくなる危険性が有るので、
                 * 一旦リリースする。
                 */
                this.mouseLeftRelease();
                this.targetWindow.alert("マウスポインタを、Webブラウザの外に移動しようとしました。\n"+
                                        "処理を終了します。");
                throw new Error("Illegal position :" + x + "/" + y);
            }
        }

        this.updatePosition();
        this.moveX = x;
        this.moveY = y;
        this.send("MouseMove," + (this.documentLeftOnScreen + x) + "," + (this.documentTopOnScreen + y));
    };

    /**
     * マウスカーソルを、現地点から指定の移動量だけ移動させます.
     * 移動量は、ピクセルです。左方向または上方向への移動は、負数で指定します。
     * 内部で、指定の値をスクリーン上の位置に変換して、マウスカーソルを移動させます。
     * @param {Number} x X方向のピクセル。左方向への移動は、負数で指定します
     * @param {Number} y Y方向のピクセル。上方向への移動は、負数で指定します
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMoveFromHere = function (x, y) {

        this.mouseMove(this.moveX + x, this.moveY + y);
    };

    /**
     * マウスカーソルを、tagNameとindexで指定された要素の上に移動します.
     * indexは、以下のコードで返されるNodeListのindexです。
     * <pre>document.getElementsByTagName(tagName);</pre>
     * @param {String} tagName マウスを移動する要素タグ名称
     * @param {Number} index マウスを移動する要素インデックス
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMoveToTagNameAt = function (tagName, index, direction, x, y) {

        var targetElements = this.targetDocument.getElementsByTagName(tagName);
        if (index >= targetElements.length) {
            Log.error("Illegal argment. tagName:" + tagName + ' index:' + index);
            throw new Error("Illegal argment. tagName:" + tagName + ' index:' + index);
            //return;
        }
        this.mouseMoveTo(targetElements.item(index), direction, x, y);
    };

    /**
     * マウスカーソルを、idで指定された要素の上に移動します.
     * @param {String} id マウスを移動する要素のID
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMoveToId = function (id, direction, x, y) {

        var targetElement = this.targetDocument.getElementById(id);
        if (!targetElement) {
            Log.error("Illegal id:" + id);
            throw new Error("Illegal id:" + id);
            //return;
        }
        this.mouseMoveTo(targetElement, direction, x, y);
    };

    /**
     * マウスカーソルを、nameで指定された要素の上に移動します.
     * ドキュメント内に、同じnameの要素が複数存在する場合、先に見つけた要素に移動します。
     * @param {String} name マウスを移動する要素のname
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMoveToName = function (name, index, direction, x, y) {

        if ('undefined' == typeof index) {

            index = 0;
        }

        var targetElement = this.targetDocument.getElementsByName(name);
        if (!targetElement || index >= targetElement.length) {
            Log.error("Illegal name:" + name);
            throw new Error("Illegal name:" + name);
            //return;
        }
        this.mouseMoveTo(targetElement.item(index), direction, x, y);
    };

    /**
     * マウスカーソルを、指定の要素の上に移動します.
     * @param {Element} targetElement マウスを移動する要素
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseMoveTo = function(targetElement, direction, x, y) {

        if ('undefined' == typeof direction) {

            direction = "Center";
        }

        if ('undefined' == typeof x) {

            x = 0;
        }

        if ('undefined' == typeof y) {

            y = 0;
        }

        if (!targetElement) {
            Log.error("Argument is null.");
            throw new Error("Argument is null.");
            //return;
        }
        var htmlPosition = HtmlPosition.createHtmlPosition(targetElement);
        var point = htmlPosition.getPoint(direction);
        this.mouseMove(point.x + x, point.y + y);
    };

    /**
     * マウスの左ボタンを押すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseLeftPress = function (isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.send("MousePress,TRUE,FALSE,FALSE," + getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey));
    };

    /**
     * マウスの左ボタンを放すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseLeftRelease = function (isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.send("MouseRelease,TRUE,FALSE,FALSE," + getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey));
    };

    /**
     * マウスの中ボタンを押すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseCenterPress = function (isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.send("MousePress,FALSE,TRUE,FALSE," + getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey));
    };

    /**
     * マウスの中ボタンを放すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseCenterRelease = function (isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.send("MouseRelease,FALSE,TRUE,FALSE," + getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey));
    };

    /**
     * マウスの右ボタンを押すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseRightPress = function (isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.send("MousePress,FALSE,FALSE,TRUE," + getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey));
    };

    /**
     * マウスの右ボタンを放すイベントを発生させます。
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseRightRelease = function (isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.send("MouseRelease,FALSE,FALSE,TRUE," + getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey));
    };

    JSRobot.prototype.mouseWheel = function (wheelAmt) {

        this.send("MouseWheel," + wheelAmt);
    };

    /**
     * マウスの左ボタンをクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseLeftClick = function (isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.send("MouseClick,TRUE,FALSE,FALSE," + getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey));
    };

    /**
     * マウスの中ボタンをクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseCenterClick = function (isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.send("MouseClick,FALSE,TRUE,FALSE," + getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey));
    };

    /**
     * マウスの右ボタンをクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseRightClick = function (isCtrlKey, isShiftKey, isAltKey, isMetaKey) {

        this.send("MouseClick,FALSE,FALSE,TRUE," + getModifyText(isCtrlKey, isShiftKey, isAltKey, isMetaKey));
    };

    /**
     * マウスの左ボタンをダブルクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseLeftDblclick = function () {

        this.mouseLeftClick();
        this.mouseLeftClick();
    };

    /**
     * マウスの中ボタンをダブルクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseCenterDblclick = function () {

        this.mouseCenterClick();
        this.mouseCenterClick();
    };

    /**
     * マウスの右ボタンをダブルクリックするイベントを発生させます.
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.mouseRightDblclick = function () {

        this.mouseRightClick();
        this.mouseRightClick();
    };

    /**
     * 指定のIDの要素をクリックして、フォーカスを移動させます。
     * @param id フォーカスを移動する要素のID
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.focusToId = function(id) {

        this.mouseMoveToId(id);
        this.mouseLeftClick();
    };

    /**
     * idで指定された要素にtextの内容を入力するイベントを発生させます。
     * このメソッドは、Firefoxでは、正しく動作しません。
     * @param id textの内容を入力する要素のID
     * @param text 入力する文字列
     * @memberOf com.yscjp.webtools.JSRobot
     */
    JSRobot.prototype.inputTextToId = function(id, text) {

        this.mouseMoveToId(id);
        this.mouseLeftClick();
        this.inputText(text);
    };

    JSRobot.prototype.windowCapture = function(title) {

        var captureWindow = this.targetWindow;

        if (!captureWindow) {
            captureWindow = window;
        }

        var x;
        var y;
        var height;
        var width;

        if ('undefined' == typeof captureWindow.screenX) {

            x = 0;
            y = 0;
            height = window.screen.height;
            width = window.screen.width;

        } else {

            x = captureWindow.screenX;
            y = captureWindow.screenY;
            height = captureWindow.outerHeight;
            width = captureWindow.outerWidth;
        }

        title = title == null ? "" : title;

        Log.debug("JSRobot#windowCapture x:" + x + " y:" + y + " width:" + width + " height:" + height);

        this.capture(x, y, width, height, title);
    };

    JSRobot.prototype.capture = function(x, y, width, height, title) {

        this.send("ScreenCapture," + x + "," + y + "," + width + "," + height + "," + title);
    };
})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.webtools.JSRobot, "com.yscjp.webtools.JSRobot");
