/**
 * Web自動テストツール.
 * Webアプリケーションを、JavaのRobotクラスが生成するマウスやキーボードのイベントで、
 * 自動的にテストするツールです。
 * Java Appletが動作する環境が必要です。
 *
 * Version 00.12.00.00
 *
 * このプログラムのライセンスについて。
 * このプログラムは、MITライセンス(The MIT License)の元で公開します。
 * Released under the MIT Licenses.
 *
 * ライセンスの原文は、以下のURLを参照してください。
 * http://www.yscjp.com/licenses/MIT_LICENSE.txt
 *
 * Copyright (c) 2011-2013 Yagyu Teru, http://yscjp.com/
 *
 */

var com;
if (!com) com = {};
if (!com.yscjp) com.yscjp = {};
if (!com.yscjp.jsapp) com.yscjp.jsapp = {};
if (!com.yscjp.jsapp.webtest) com.yscjp.jsapp.webtest = {};

var __loadedContentsListener__ = null;
var __unloadContentsListener__ = null;
var __mouseOutListener__ = null;

com.yscjp.jsapp.webtest.Webtest = function () {};
(function() {

    var Webtest = com.yscjp.jsapp.webtest.Webtest;

    var Log = com.yscjp.jsapp.util.Log;
    var JSEvent = com.yscjp.jsapp.JSEvent;
    var XmlUtil = com.yscjp.jsapp.util.XmlUtil;
    var ArgChk = com.yscjp.jsapp.util.ArgChk;
    var UA = com.yscjp.jsapp.util.UA;

    var Autorun = com.yscjp.jsapp.autorun.Autorun;
    var EventCapture = com.yscjp.jsapp.autorun.EventCapture;

    var loadContentsWindow = null;
    var loadContentsURL = null;

    /*
     * コンテンツのロードを待つ時間（ミリ秒）
     * この時間を待ってロードが完了しない場合、ダイアログを表示しシナリオを中断します。
     */
    var loadTimeout = 60000;
    var loatTimer = null;

    var newWindow = null;
    var delayTime = 100;

    var captureNow = false;

    var jsRobot = null;

    /*
     * Windowからカーソルが外れたら、シナリオを自動終了する処理を、スキップするかどうかの指示。
     */
    var isOutStopSkip = false;

    /*
     * プライベートメソッドを管理する内部クラス。
     * トレースを取得する為に、一元管理していつ必要があるため、このような実装方法としている。
     */
    var __PFunc = function () {};

    var surveilance = null;

    var isOpen = false;

    var scenariolistBasePath = "";
    var scenarioBasePath = "";

    function initializeWebtest(serverPortFunc) {

        jsRobot = new com.yscjp.webtools.JSRobot(serverPortFunc);
        return jsRobot;
    }

    /**
     * テストケースを実行します.
     * @param {String} testcaseUrl 実行するテストシナリオのURL
     * @param {Number} _delayTime シナリオを実行するミリ秒単位の間隔。
     * @param {boolean} _isOutStop カーソルがWindowの外に出たら、自動終了するかどうかのフラグ。
     * @memberOf com.yscjp.jsapp.webtest.Webtest
     */
    function execTestCase(testcaseUrl, _delayTime, _loadTimeout, isUseTargetwinTimer, isOutStop, _scenariolistBasePath) {

        scenariolistBasePath = _scenariolistBasePath;
        var xmlDoc;
        var url = scenariolistBasePath + testcaseUrl + '?' + new Date().getTime();
        scenarioBasePath = scenariolistBasePath;
        var slashIndex = url.lastIndexOf("/");
        if (-1 != slashIndex) {

            scenarioBasePath = url.substring(0, slashIndex + 1);
        }
        Log.debug("scenarioBasePath:" + scenarioBasePath);
        Log.debug("scenario url:" + url);
        var xmlLS = new com.yscjp.jsapp.tools.XmlLS();
        xmlDoc = xmlLS.perseFromURI(url);
        if (null == xmlDoc) {

            var errorMsg = 'Http request error. ' +
            xmlLS.getResultCode() + ':' +
            xmlLS.getResultText() + '' +
            url;
            Log.error(errorMsg);
            throw errorMsg;
        }

        __PFunc.execTestCase(xmlDoc, _delayTime, _loadTimeout, isUseTargetwinTimer, isOutStop, scenarioBasePath);
    }


    __PFunc.execTestCase = function (xmlDoc, _delayTime, _loadTimeout, isUseTargetwinTimer, isOutStop, scenarioBasePath) {

        delayTime = ArgChk.notUndefined(_delayTime, 100);
        loadTimeout = _loadTimeout;

        __PFunc.webtestExit();

        if (isOutStop) {

            __mouseOutListener__ = __PFunc.mouseOutListener;
        }

        captureNow = false;

        //Log.debug("xmlDoc:" + xmlDoc);

        Autorun.initialize(xmlDoc,
                           delayTime,
                           isUseTargetwinTimer,
                           isOutStop,
                           __PFunc.webtestExit,
                           scenarioBasePath);
        Autorun.run();
    };


    __PFunc.mouseOutListener = function (screenX, screenY) {

        if (isOutStopSkip) {

            Log.debug("OutStop Skip");

            /*
             * 一時的に、自動終了を回避したケース。
             * このリスナが呼ばれたということは、シナリオ開始時には自動終了がOnとなっていたはずなので、
             * 回避するのは一回だけ。
             */
            isOutStopSkip = false;
            return;
        }

        var documentTopOnScreen = jsRobot.getDocumentTopOnScreen();
        var documentLeftOnScreen = jsRobot.getDocumentLeftOnScreen();

        //if (Log.isDebug) {

            Log.debug("screenX:" + screenX +
                      " documentLeftOnScreen:" + documentLeftOnScreen +
                      " screenY:" + screenY +
                      " documentTopOnScreen:" + documentTopOnScreen);
        //}

        if (documentLeftOnScreen <= screenX && documentTopOnScreen <= screenY ) {

            var innerWidth = com.yscjp.jsapp.util.UA.getInnerWidth(loadContentsWindow);
            var innerHeight = com.yscjp.jsapp.util.UA.getInnerHeight(loadContentsWindow);

            Log.debug("innerWidth:" + innerWidth + " innerHeight:" + innerHeight);

            if ((documentLeftOnScreen + innerWidth) >= screenX && (documentTopOnScreen + innerHeight) >= screenY ) {

                return;
            }
        }

        Autorun.cancel();

        com.yscjp.jsapp.ui.Billboard.viewMessage("Window外にカーソルが移動しました。\nシナリオを中止します。");
        //alert("Window外にカーソルが移動しました。\nシナリオを中止します。");
    };

    __PFunc.webtestExit = function () {

        __PFunc.loadTimeoutCancel();
        __loadedContentsListener__ = null;
        __unloadContentsListener__ = null;
        __mouseOutListener__ = null;
    };

    __PFunc.loadListener = function () {

        jsRobot.targetDocument = newWindow.document;
    };

    __PFunc.unloadListener = function () {
    };

    function loadTimeoutListener () {

        if (loadContentsWindow && !loadContentsWindow.closed) {

            com.yscjp.jsapp.util.UA.stopLoad(loadContentsWindow);
            loadContentsWindow.alert("ページがロードできませんでした。");

        } else {

            com.yscjp.jsapp.util.UA.stopLoad();
            alert("ページがロードできませんでした。");
        }

        Autorun.cancel();
    };

    __PFunc.setLoadTimeout = function () {

        loatTimer = setTimeout("com.yscjp.jsapp.webtest.Webtest.loadTimeoutListener()", loadTimeout);
    };

    __PFunc.loadTimeoutCancel = function () {

        if (loatTimer) {

            clearTimeout(loatTimer);
            loatTimer = null;
        }
    };

    __PFunc.surveilanceLoadedHdl = function () {

        Log.debug("---------- __PFunc.surveilanceLoadedHdl ----------");

//        var dialogWrap = document.getElementById('inhibit_dialog_wrap');
//        var closeWrap = document.getElementById('inhibit_close_wrap');
//
//        if (!dialogWrap || !dialogWrap.checked) {
//
//            /*
//             * ダイアログのオーバーライド
//             * シナリオ実行の自動終了をスキップするように変更
//             */
//            var __outStopSkip__ = outStopSkip;
//            loadContentsWindow.__alert__ = loadContentsWindow.alert;
//            loadContentsWindow.alert = function (msg) {
//                __outStopSkip__();
//                this.__alert__(msg);
//            };
//            loadContentsWindow.__confirm__ = loadContentsWindow.confirm;
//            loadContentsWindow.confirm = function (msg) {
//                outStopSkip();
//                return this.__confirm__(msg);
//            };
//        }
//
//        if (!closeWrap || !closeWrap.checked) {
//
//            loadContentsWindow.__close__ = loadContentsWindow.close;
//            loadContentsWindow.close = function () {
//                isOpen = false;
//                this.__close__();
//            };
//        }

        if (loadContentsWindow) {

            if ("undefined" == typeof loadContentsWindow.onmouseout) {

                if ("undefined" != typeof loadContentsWindow.document.onmouseout) {

                    JSEvent.addEventListener(loadContentsWindow.document, 'mouseout', __PFunc.surveilanceMouseOutHdl, false);
                }

            } else {

                JSEvent.addEventListener(loadContentsWindow, 'mouseout', __PFunc.surveilanceMouseOutHdl, false);
            }
        }

        if (__loadedContentsListener__) {
            __loadedContentsListener__();
        }

    };

    __PFunc.surveilanceUnloadHdl = function () {

        Log.debug("---------- __PFunc.surveilanceUnloadHdl ----------");

        if (__unloadContentsListener__) {
            __unloadContentsListener__();
        }
    };

    __PFunc.surveilanceMouseOutHdl = function (e) {

        Log.debug("---------- __PFunc.surveilanceMouseOutHdl ----------");

        if (__mouseOutListener__) {
            __mouseOutListener__(e.screenX, e.screenY);
        }
    };

    /**
     * コンテンツをロードします.
     * @param {String} loadURL ロードするコンテンツのURL
     * @param {String} openSetting オープンする window の設定内容
     * @memberOf com.yscjp.jsapp.webtest.Webtest
     */
    function loadContent(loadURL, openSetting) {

        if (com.yscjp.jsapp.util.JSString.startsWith(loadURL, "http://") ||
            com.yscjp.jsapp.util.JSString.startsWith(loadURL, "/")) {

            loadContentsURL = loadURL;

        } else {

            loadContentsURL = scenarioBasePath + loadURL;
        }

        Log.debug("loadContentsURL:" + loadContentsURL);

        if (!loadContentsWindow || loadContentsWindow.closed) {

            Log.debug("window.open");

            __loadedContentsListener__ = contentLoaded;
//            __loadedContentsListener__ = __PFunc.setURL;
            loadContentsWindow = window.open(loadContentsURL, "_blank", openSetting);
            if (-1 != openSetting.search(/channelmode=yes,/) && "IE" != UA.getUA()) {

                loadContentsWindow.moveTo(0,0);
                loadContentsWindow.resizeTo(screen.availWidth,screen.availHeight);
            }

            isOpen = true;

            surveilance = new com.yscjp.jsapp.tools.WindowSurveillance(loadContentsWindow,
                    100,
                    __PFunc.surveilanceLoadedHdl,
                    __PFunc.surveilanceUnloadHdl);
            surveilance.start();

        } else {

            Log.debug("window not open");

            __PFunc.webtestExit();
            __PFunc.setURL();
        }

        loadContentsWindow.focus();
        Autorun.setNonSetTimeout();
//        Autorun.setWaiting(true);
        __PFunc.setLoadTimeout();
    }

    __PFunc.setURL = function () {

        __loadedContentsListener__ = contentLoaded;
        loadContentsWindow.location.href = loadContentsURL;
    };

    __PFunc.execOnload = function () {

        Autorun.setTargetWindow(loadContentsWindow);
        jsRobot.setDocument(loadContentsWindow);
        Autorun.setWaiting(false);
        __PFunc.loadTimeoutCancel();
    };

    __PFunc.execOnunload = function () {

        if (isOpen && Autorun.isRunning()) {

            Autorun.setWaiting(true);
            __PFunc.setLoadTimeout();
        }
    };

    function contentLoaded() {

        __loadedContentsListener__ = __PFunc.execOnload;
        __unloadContentsListener__ = __PFunc.execOnunload;

        var contentWindow = loadContentsWindow;

        Autorun.setTargetWindow(contentWindow);
        jsRobot.setDocument(contentWindow);
        if (jsRobot.updatePosition()) {

            /*
             * 引数なしでJSRobot#updatePositionを呼んで、trueが返った場合、
             * 画面の位置合わせが終了したので、これ以上の処理は必要ない。
             */
//            Autorun.setWaiting(false);
            __PFunc.loadTimeoutCancel();
            setTimeout('com.yscjp.jsapp.autorun.Autorun.execute()', Autorun.delayTime);
            return;
        }

        /*
         * 古いFirefoxやWebKit
         */
        JSEvent.addEventListener(contentWindow.document, 'mousemove', __PFunc.mouseMoved, false);
        var windowTop = contentWindow.screenY;
        var windowLeft = contentWindow.screenX;
        if ("undefined" != typeof contentWindow.screenY) {

            windowTop = contentWindow.screenY;
            windowLeft = contentWindow.screenX;

        } else if ("undefined" != typeof contentWindow.screenTop) {

            windowTop = contentWindow.screenTop;
            windowLeft = contentWindow.screenLeft;

        } else {

            Log.debug('Window position unknown.');
            return;
        }

        var innerWidth = com.yscjp.jsapp.util.UA.getInnerWidth(contentWindow);
        var innerHeight = com.yscjp.jsapp.util.UA.getInnerHeight(contentWindow);

        Log.debug('windowTop:' + windowTop + ' windowLeft:' + windowLeft + ' innerHeight:' + innerHeight + ' innerWidth:' + innerWidth);
        jsRobot.mouseMove(windowLeft + (innerWidth / 2), windowTop + (innerHeight / 2));
        //jsRobot.mouseLeftClick();
    }

    __PFunc.mouseMoved = function (e) {

        e = JSEvent.createEvent(e);
        Log.debug('screenX:' + e.screenX +
                 ' screenY:' + e.screenY +
                 ' clientX:' + e.clientX +
                 ' clientY:' + e.clientY);
        jsRobot.updatePosition(e.screenY - e.clientY, e.screenX - e.clientX);
        //isLoaded = true;

        var contentWindow = loadContentsWindow;
        JSEvent.removeEventListener(contentWindow.document, 'mousemove', __PFunc.mouseMoved, false);

        if (captureNow) {

            EventCapture.setTargetWindow(loadContentsWindow);

            /*
             * イベントキャプチャ中
             * ここで直接captureInit()を呼ぶと、mouseClicked(e)が呼ばれた原因となった
             * イベントで、mouseclickListenerが呼ばれてしまうので、それをさけるために
             * 一定時間後に呼び出すようにする。
             */
            setTimeout('com.yscjp.jsapp.autorun.EventCapture.captureInit()', delayTime);

        } else {

            /*
             * テストケース再開
             */
//            Autorun.setWaiting(false);
            __PFunc.loadTimeoutCancel();
            setTimeout('com.yscjp.jsapp.autorun.Autorun.execute()', Autorun.delayTime);
        }
    };

    function capture(loadURL, openSetting, modifyEventCapture, pressReleaseCapture, labelclick) {

        captureNow = true;

        EventCapture.initialize(modifyEventCapture,
                                pressReleaseCapture,
                                labelclick);

        newWindow = window.open(loadURL, null, openSetting);
        if (-1 != openSetting.search(/channelmode=yes,/)) {

            newWindow.moveTo(0,0);
            newWindow.resizeTo(screen.availWidth,screen.availHeight);
        }
        newWindow.focus();
        EventCapture.setTargetWindow(newWindow);

        /*
         * イベントキャプチャ中
         * ここで直接captureInit()を呼ぶと、mouseClicked(e)が呼ばれた原因となった
         * イベントで、mouseclickListenerが呼ばれてしまうので、それをさけるために
         * 一定時間後に呼び出すようにする。
         */
        setTimeout('com.yscjp.jsapp.autorun.EventCapture.captureInit()', delayTime);
    }

    function manualTest(loadURL, openSetting, modifyEventCapture, pressReleaseCapture, labelclick) {

        captureNow = true;

        EventCapture.initialize(modifyEventCapture,
                                pressReleaseCapture,
                                labelclick,
                                __PFunc.captureOnloadHdl,
                                __PFunc.captureOnunloadHdl);

        newWindow = window.open(loadURL, null, openSetting);
        if (-1 != openSetting.search(/channelmode=yes,/)) {

            newWindow.moveTo(0,0);
            newWindow.resizeTo(screen.availWidth,screen.availHeight);
        }
        newWindow.focus();
        EventCapture.setTargetWindow(newWindow);

        /*
         * イベントキャプチャ中
         * ここで直接captureInit()を呼ぶと、mouseClicked(e)が呼ばれた原因となった
         * イベントで、mouseclickListenerが呼ばれてしまうので、それをさけるために
         * 一定時間後に呼び出すようにする。
         */
        setTimeout('com.yscjp.jsapp.autorun.EventCapture.manualTestInit()', delayTime);
    }

    function captureEnd() {

        captureNow = false;
        var capturetext = document.getElementById('capturetext');
        capturetext.value = EventCapture.captureEnd();

    }

    function captureExec(_delayTime, _loadTimeout, isUseTargetwinTimer, isOutStop, _scenarioBasePath) {

        scenariolistBasePath = _scenarioBasePath;
        scenarioBasePath = _scenarioBasePath;

        var xmlDoc = XmlUtil.perseFromString(document.getElementById('capturetext').value);
        if (null == xmlDoc) {

            var errorMsg = 'Http request error. ' +
            xmlLS.getResultCode() + ':' +
            xmlLS.getResultText() + '' +
            url;
            Log.error(errorMsg);
            throw errorMsg;
        }

        __PFunc.execTestCase(xmlDoc, _delayTime, _loadTimeout, isUseTargetwinTimer, isOutStop, scenarioBasePath);
    }

    __PFunc.captureOnloadHdl = function () {

        //Webtest.imageCapture(newWindow.document.title + "_Onload");
        setTimeout("com.yscjp.jsapp.webtest.Webtest.imageCapture('" + newWindow.document.title + "_Onload')", 100);
    };

    __PFunc.captureOnunloadHdl = function () {

        Webtest.imageCapture(newWindow.document.title + "_Onunload");
    };

    com.yscjp.jsapp.util.StackTrace.insert(__PFunc, "com.yscjp.jsapp.webtest.Webtest.__PFunc");

    function imageCapture(title, top, left, height, width) {

        if ("undefined" == typeof top) {

            if (document.getElementById("imageCaptureSizeSpecify").checked) {

                top = document.getElementById("capture_top").value;
                left = document.getElementById("capture_left").value;
                height = document.getElementById("capture_height").value;
                width = document.getElementById("capture_width").value;

            } else {

                jsRobot.windowCapture(title);
                return;
            }
        }

        jsRobot.capture(left, top, width, height, title);
    }

    function outStopSkip() {

        isOutStopSkip = true;
    }

    function closeLoadContentsWindow() {

        if (loadContentsWindow && !loadContentsWindow.closed) {

            loadContentsWindow.close();
        }

        Autorun.cancel();
    }

    Webtest.initializeWebtest = initializeWebtest;
    Webtest.execTestCase = execTestCase;
    Webtest.loadContent = loadContent;
    Webtest.loadTimeoutListener = loadTimeoutListener;
    Webtest.contentLoaded = contentLoaded;
    //Webtest.mouseMoved = mouseMoved;
    Webtest.capture = capture;
    Webtest.manualTest = manualTest;
    Webtest.captureEnd = captureEnd;
    Webtest.captureExec = captureExec;
    Webtest.imageCapture = imageCapture;
    Webtest.outStopSkip = outStopSkip;
    Webtest.closeLoadContentsWindow = closeLoadContentsWindow;

})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.webtest.Webtest,
                                       "com.yscjp.jsapp.webtest.Webtest");
