/**
 * Webユーティリティ
 * Webアプリケーションを作成するうえで、便利なユーティリティ。
 * ある程度のWebブラウザ間の違いは、この中で吸収します。
 *
 * Version 00.15.00.00
 *
 * このプログラムのライセンスについて。
 * このプログラムは、MITライセンス(The MIT License)の元で公開します。
 * Released under the MIT Licenses.
 *
 * ライセンスの原文は、以下のURLを参照してください。
 * http://www.yscjp.com/licenses/MIT_LICENSE.txt
 *
 * Copyright (c) 2010-2013 Yagyu Teru, http://yscjp.com/
 *
 */
var com;
if (!com) com = {};
if (!com.yscjp) com.yscjp = {};
if (!com.yscjp.jsapp) com.yscjp.jsapp = {};
if (!com.yscjp.jsapp.util) com.yscjp.jsapp.util = {};

com.yscjp.jsapp.util.JS = function () {};
(function() {

    var JS = com.yscjp.jsapp.util.JS;
    JS.executeListener = function (listener) {
        return listener();
    };

})();


com.yscjp.jsapp.util.UA = function () {};
(function() {

    var UA = com.yscjp.jsapp.util.UA;

    var isInitialized = false;
    var isFixedSupport = true;
    var emPx = 0;

    function init() {

        var div01 = document.createElement("div");
        div01 = document.body.insertBefore(div01, document.body.firstChild);
        var firstTop = div01.offsetTop;
        var div02 = document.createElement("div");
        div02.style.border = "none";
        div02.style.padding = "0";
        div02.style.height = "1em";
        div02.style.position = "fixed";
        div02 = document.body.insertBefore(div02, document.body.firstChild);
        var secondTop = div01.offsetTop;
        if (firstTop != secondTop) {

            isFixedSupport = false;
        }
        emPx = div02.offsetHeight;

        document.body.removeChild(div01);
        document.body.removeChild(div02);
    }

    /**
     * Webブラウザが、スタイルシートの position: fixed をサポートするかを返します.
     * @return position: fixed をサポートする場合 true、サポートしない場合 false
     * @memberOf com.yscjp.jsapp.util.UA
     */
    UA.isFixedSupport = function () {

        if (!isInitialized) {

            init();
        }

        return isFixedSupport;
    };

    /**
     * 1em のピクセル数を返します.
     * @return 1em のピクセル数
     * @memberOf com.yscjp.jsapp.util.UA
     */
    UA.getEmPx = function () {

        if (!isInitialized) {

            init();
        }

        return emPx;
    };

    /**
     * このリクエストに付加されたクエリから、指定のキーの値を取得し返します。
     * クエリに該当のキーが複数存在した場合、前に指定されているものが有効となります。
     * @param key [string] クエリから値を取得するキー
     * @return keyに該当する値。クエリが存在しないか、クエリにキーが存在しない場合、null
     * @memberOf com.yscjp.jsapp.util.UA
     */
    UA.getQueryValue = function (key) {

        var value = null;
        var query = window.location.search;
        if (0 != query.length) {

            /*
             * 先頭の'?'を削除します。
             */
            query = query.replace(/^\?/, "");

            /*
             * &で分割します。
             */
            var queries = query.split('&');
            for (var i = 0; i < queries.length; i++) {

                var reg = new RegExp("^" + key + "=(.*)");
                var values = queries[i].match(reg);
                if (null != values && 2 == values.length) {

                    value = values[1];
                    break;
                }
            }
        }

        return value;
    };

    /**
     * このリクエストに付加されたクエリから、指定のキーの値を取得し返します。
     * クエリに該当のキーが複数存在した場合、全てを有効とします。
     * @param key [string] クエリから値を取得するキー
     * @return keyに該当する値を格納した配列。クエリが存在しないか、クエリにキーが存在しない場合、空の配列
     * @memberOf com.yscjp.jsapp.util.UA
     */
    UA.getQueryValues = function (key) {

        var values = new Array();
        var query = window.location.search;
        if (0 != query.length) {

            /*
             * 先頭の'?'を削除します。
             */
            query = query.replace(/^\?/, "");

            /*
             * &で分割します。
             */
            var queries = query.split('&');
            for (var i = 0; i < queries.length; i++) {

                var reg = new RegExp("^" + key + "=(.*)");
                var _values_ = queries[i].match(reg);
                if (null != _values_ && 2 == _values_.length) {

                    values.push(_values_[1]);
                }
            }
        }

        return values;
    };

    /**
     * 指定されたWindowの、表示領域の横幅をピクセル単位で返します。
     * @param targetWindow [Window] 横幅を取得するWindow。指定されていない場合は、window
     * @return 指定されたWindowの、表示領域の横幅
     * @memberOf com.yscjp.jsapp.util.UA
     */
    UA.getInnerWidth = function (targetWindow) {

        if ("undefined" == typeof targetWindow) {

            targetWindow = window;
        }

        if ("undefined" != typeof targetWindow.innerWidth) {

            return targetWindow.innerWidth;
        }

        var targetDocument = targetWindow.document;

        if ("undefined" != typeof targetDocument.documentElement) {

            if ("undefined" != typeof targetDocument.documentElement.clientWidth) {

                return targetDocument.documentElement.clientWidth;
            }
        }

        if ("undefined" != typeof targetDocument.body) {

            if ("undefined" != typeof targetDocument.body.clientWidth) {

                /*
                 * この属性は、Windowの表示領域とはかなり異なるサイズとなるので、
                 * 実際に使えるかどうかは不明。
                 */
                return targetDocument.body.clientWidth;
            }
        }

        return 0;
    };

    /**
     * 指定されたWindowの、表示領域の高さをピクセル単位で返します。
     * @param targetWindow [Window] 高さを取得するWindow。指定されていない場合は、window
     * @return 指定されたWindowの、表示領域の高さ
     * @memberOf com.yscjp.jsapp.util.UA
     */
    UA.getInnerHeight = function (targetWindow) {

        if ("undefined" == typeof targetWindow) {

            targetWindow = window;
        }

        if ("undefined" != typeof targetWindow.innerHeight) {

            return targetWindow.innerHeight;
        }

        var targetDocument = targetWindow.document;

        if ("undefined" != typeof targetDocument.documentElement) {

            if ("undefined" != typeof targetDocument.documentElement.clientHeight) {

                return targetDocument.documentElement.clientHeight;
            }
        }

        if ("undefined" != typeof targetDocument.body) {

            if ("undefined" != typeof targetDocument.body.clientHeight) {

                /*
                 * この属性は、Windowの表示領域とはかなり異なるサイズとなるので、
                 * 実際に使えるかどうかは不明。
                 */
                return targetDocument.body.clientHeight;
            }
        }

        return 0;
    };

    /**
     * ページの読み込みを中止します。
     * @param targetWindow [Window] ページの読み込みを中止Window。指定されていない場合は、window
     * @memberOf com.yscjp.jsapp.util.UA
     */
    UA.stopLoad = function (targetWindow) {

        if ("undefined" == typeof targetWindow) {

            targetWindow = window;
        }

        if (targetWindow.document.execCommand) {

            targetWindow.document.execCommand("Stop");

        } else {

            targetWindow.stop();
        }
    };

    /**
     * ユーザエージェントの識別子を返します。
     * <p>現在サポートしているUAと、返却する識別子は以下の通り。
     * <pre>Internet Explorer : IE
     * Firefox           : FF
     * Google Chrome     : GC
     * Opera             : OP
     * Safari            : SF
     * 不明              : UD</pre>
     * <p>このメソッドは実験的なので、使用には注意すること。
     * 過去のバージョンや将来のバージョンで、正しく動作しません。
     * @return 使用中のUAを現す識別子
     * @memberOf com.yscjp.jsapp.util.UA
     */
    UA.getUA = function () {

        if (window.ActiveXObject) {

            return "IE";
        }

        if (window.mozInnerScreenY) {

            /*
             * 3.6以降
             */
            return "FF";
        }

        if (window.webkitConvertPointFromNodeToPage) {

            /*
             * Webkit
             */
            if (window.IDBFactory) {

                return "GC";

            } else {

                return "SF";
            }
        }

        if (window.opera) {

            return "OP";
        }

        return "UD";    // undefined
    };

})();


/**
 * ローテーションするバッファの実装です.
 * @class 複数の面を管理し、ローテーションするバッファです.
 * @returns
 */
com.yscjp.jsapp.util.RotateBuffer = function (maxLength, maxCount) {

    this.maxLength = maxLength;
    this.maxCount = maxCount;
    this.topIndex = 0;
    this.bottomIndex = this.topIndex + this.maxCount - 1;
    this.index = 0;
    this.areas = new Array();
    for (var i = 0; i < this.maxCount; i++) {
        this.areas[i] = new Array();
    }
};
(function() {

    var RotateBuffer = com.yscjp.jsapp.util.RotateBuffer;
    RotateBuffer.prototype.put = function (msg) {
        var putArea = this.areas[this.index];
        if (this.maxLength <= putArea.length) {
            this.index++;
            if (this.bottomIndex < this.index) {
                this.areas[this.topIndex] = null;
                this.topIndex++;
                this.bottomIndex++;
                this.areas[this.bottomIndex] = new Array();
            }
            putArea = this.areas[this.index];
        }
        putArea.push(msg);
    };

    RotateBuffer.prototype.print = function (listener) {

        for (var i = this.topIndex; i <= this.bottomIndex; i++) {
            var area = this.areas[i];
            for (var j = 0; j < area.length; j++) {
                listener(area[j]);
            }
        }
    };

    RotateBuffer.prototype.clear = function () {

        this.topIndex = 0;
        this.bottomIndex = this.topIndex + this.maxCount - 1;
        this.index = 0;
        //this.areas = new Array();
        for (var i = 0; i < this.maxCount; i++) {
            this.areas[i] = new Array();
        }
    };

})();

/**
 * ログ出力の管理を行います.
 * @class コンソールへログの出力を行います.
 * <p>window.console.logが存在し無い場合は、何もしません。
 * <p>TRACE・DEBUG・INFO・WARNING・ERROR・FETAL・NONEの各モードをサポートし、デフォルトのモードはINFOです。
 * デフォルトの状態では、info()・warning()・error()・fetal()の各メソッドで出力されたログを表示します。
 * モードの変更は、プログラムの中に以下のコードを記述することで変更できます。
 * <pre>com.yscjp.jsapp.util.Log.mode=com.yscjp.jsapp.util.Log.MODE_DEBUG</pre>
 * <p>上記例では、debug()メソッドにより出力されたログを表示させます。
 * <p>また、プログラムの変更が出来無い場合は、URLに以下のクエリを付加することでも変更できます。
 * <pre>?com.yscjp.jsapp.util.Log.mode=0</pre>
 * <p>上記例では、trace()メソッドにより出力されたログを表示させます。
 * TRACEが0で、DEBUGが1....FETALが5・NONEが9となります。
 */
com.yscjp.jsapp.util.Log = function () {};
(function() {

    var UA = com.yscjp.jsapp.util.UA;
    var RotateBuffer = com.yscjp.jsapp.util.RotateBuffer;
    var Log = com.yscjp.jsapp.util.Log;

    /**
     * トレースログの設定。
     * 本来このあたりは外部設定により行うが、とりあえずは固定値。
     */
    var traceBuffer = new RotateBuffer(200, 2);
    var applicationBuffer = new RotateBuffer(1000, 10);

    /**
     * ログの出力モードでトレースを表す定数(0)です.
     * @type Number
     * @memberOf com.yscjp.jsapp.util.Log
     */
    Log.MODE_TRACE = 0;
    Log.MODE_DEBUG = 1;
    Log.MODE_INFO = 2;
    Log.MODE_WARNING = 3;
    Log.MODE_ERROR = 4;
    Log.MODE_FETAL = 5;
    Log.MODE_NONE = 9;
    Log.mode = Log.MODE_INFO;

    var query = window.location.search;
    if (0 != query.length) {

        /*
         * 先頭の'?'を削除します。
         */
        query = query.replace(/\?/, "");

        /*
         * &で分割します。
         */
        var queries = query.split('&');
        for (var i = 0; i < queries.length; i++) {

            var mode = queries[i].match(/com.yscjp.jsapp.util.Log.mode=(\d)/);
            if (null != mode && 2 == mode.length) {

                Log.mode = mode[1];
            }
        }
    }

    var isConsoleOut = UA.getQueryValue("com.yscjp.jsapp.util.Log.consoleOut");
    if (isConsoleOut && "false" == isConsoleOut.toLowerCase()) {

        isConsoleOut = false;

    } else {

        isConsoleOut = true;
    }

    var isTraceTime = UA.getQueryValue("com.yscjp.jsapp.util.Log.traceTime");
    if (isTraceTime && "true" == isTraceTime.toLowerCase()) {

        isTraceTime = true;

    } else {

        isTraceTime = false;
    }

    function getTime() {

        var time = new Date();
        var hours = "" + (time.getHours() + 100);
        var minutes = "" + (time.getMinutes() + 100);
        var seconds = "" + (time.getSeconds() + 100);
        var milliSec = "" + (time.getMilliseconds() + 1000);

        return hours.substring(1) + ":" +
               minutes.substring(1) + ":" +
               seconds.substring(1) + "." +
               milliSec.substring(1) + " ";
    }

    function log(msg) {

        msg = getTime() + msg;
        applicationBuffer.put(msg);

        if (isConsoleOut) {

            if (!window.console) {
                return;
            }
            if (!window.console.log) {
                return;
            }
            console.log(msg);
        }
    }

    function putTrace(msg) {

        if (isTraceTime) {

            msg = getTime() + msg;
        }

        traceBuffer.put(msg);
    }

    function printTrace(id) {

        var console = document.getElementById(id);
        if (!console) {

            return;
        }

        com.yscjp.jsapp.util.DomUtil.removeAllChild(console);

        traceBuffer.print(function (msg) {

            var div = document.createElement("div");
            var text = document.createTextNode(msg);
            div.appendChild(text);
            console.appendChild(div);
        });
    }

    function printAppLog(id) {

        var console = document.getElementById(id);
        if (!console) {

            return;
        }

        com.yscjp.jsapp.util.DomUtil.removeAllChild(console);

        applicationBuffer.print(function (msg) {

            var div = document.createElement("div");
            var text = document.createTextNode(msg);
            div.appendChild(text);
            div = console.appendChild(div);
            if (msg.match(/^.............\[TRC\] /)) {

                div.className = "com_yscjp_htmlclass_TRCLOG";

            } else if (msg.match(/^.............\[DBG\] /)) {

                div.className = "com_yscjp_htmlclass_DBGLOG";

            } else if (msg.match(/^.............\[INF\] /)) {

                div.className = "com_yscjp_htmlclass_INFLOG";

            } else if (msg.match(/^.............\[WRN\] /)) {

                div.className = "com_yscjp_htmlclass_WRNLOG";

            } else if (msg.match(/^.............\[ERR\] /)) {

                div.className = "com_yscjp_htmlclass_ERRLOG";

            } else if (msg.match(/^.............\[FTL\] /)) {

                div.className = "com_yscjp_htmlclass_FTLLOG";
            }
        });
    }

    /**
     * トレースレベルのログを出力します.
     * ログの出力モードがMODE_TRACEの場合に出力されます。
     * @param {Object} msg ログに出力するメッセージ
     * @memberOf com.yscjp.jsapp.util.Log
     */
    function trace(msg) {

        putTrace(msg);

//        if (Log.MODE_TRACE < Log.mode) {
//
//            return;
//        }
//
//        log("[TRC] " + msg);
    }

    /**
     * デバッグレベルのログを出力します.
     * ログの出力モードがMODE_DEBUG以上の場合または、デバッグフラグがtrueの場合に出力されます。
     * @param {Object} msg ログに出力するメッセージ
     * @memberOf com.yscjp.jsapp.util.Log
     */
    function debug(msg) {

        if (false == Log.isDebug && Log.MODE_DEBUG < Log.mode) {

            return;
        }

        log("[DBG] " + msg);
    }

    /**
     * インフォメーションレベルのログを出力します.
     * ログの出力モードがMODE_INFO以上の場合に出力されます。
     * @param {Object} msg ログに出力するメッセージ
     * @memberOf com.yscjp.jsapp.util.Log
     */
    function info(msg) {

        if (Log.MODE_INFO < Log.mode) {

            return;
        }

        log("[INF] " + msg);
    }

    /**
     * ワーニングレベルのログを出力します.
     * ログの出力モードがMODE_WARNING以上の場合に出力されます。
     * @param {Object} msg ログに出力するメッセージ
     * @memberOf com.yscjp.jsapp.util.Log
     */
    function warning(msg) {

        if (Log.MODE_WARNING < Log.mode) {

            return;
        }

        log("[WRN] " + msg);
    }

    /**
     * エラーレベルのログを出力します.
     * ログの出力モードがMODE_ERROR以上の場合に出力されます。
     * @param {Object} msg ログに出力するメッセージ
     * @memberOf com.yscjp.jsapp.util.Log
     */
    function error(msg) {

        if (Log.MODE_ERROR < Log.mode) {

            return;
        }

        log("[ERR] " + msg);
    }

    /**
     * 致命的エラーレベルのログを出力します.
     * ログの出力モードがMODE_FETAL以上の場合に出力されます。
     * @param {Object} msg ログに出力するメッセージ
     * @memberOf com.yscjp.jsapp.util.Log
     */
    function fetal(msg) {

        if (Log.MODE_FETAL < Log.mode) {

            return;
        }

        log("[FTL] " + msg);
    }

    /**
     * コンソールにスタックトレースを出力します.
     * @memberOf com.yscjp.jsapp.util.Log
     */
    function stackTrace() {

        if (!window.console) {

            return;
        }

        if (!window.console.trace) {

            return;
        }

        console.trace();
    }

    function clearTraceLog() {

        traceBuffer.clear();
    }

    function clearAppLog() {

        applicationBuffer.clear();
    }

    function setConsoleOut(bool) {

        isConsoleOut = bool;
    }

    Log.trace = trace;
    Log.debug = debug;
    Log.info = info;
    Log.warning = warning;
    Log.error = error;
    Log.fetal = fetal;
    Log.isDebug = false;
    Log.stackTrace = stackTrace;
    Log.printTrace = printTrace;
    Log.printAppLog = printAppLog;
    Log.clearTraceLog = clearTraceLog;
    Log.clearAppLog = clearAppLog;
    Log.setConsoleOut = setConsoleOut;

    Log.info("com.yscjp.jsapp.util.Log.mode:" + Log.mode);

})();

com.yscjp.jsapp.util.StackTrace = function () {};

(function () {

    var StackTrace = com.yscjp.jsapp.util.StackTrace;
    var isStackTrace = true;
    var isTraceOnly = false;

    var query = window.location.search;
    if (0 != query.length) {

        /*
         * 先頭の'?'を削除します。
         */
        query = query.replace(/\?/, "");

        /*
         * &で分割します。
         */
        var queries = query.split('&');
        for (var i = 0; i < queries.length; i++) {

            if ("com.yscjp.jsapp.util.StackTrace=NONE" == queries[i]) {

                isStackTrace = false;
                break;
            }

            if ("com.yscjp.jsapp.util.StackTrace.traceOnly=true" == queries[i]) {

                isTraceOnly = true;
                break;
            }
        }
    }

    var Log = com.yscjp.jsapp.util.Log;
    var deep = 100;
    var stackList = new Array(deep);
    var errorStackList = new Array();
    var index = -1;
    var isStop = false;
    var id = 1000000;

    function start(clazzName, methodName, args) {

        index++;
        var stack = new Object();
        stack.clazzName = clazzName;
        stack.methodName = methodName;
        stack.args = args;

        errorStackList.length = 0;
        stackList[index] = stack;

        isStop = false;
    }

    function end() {

        if (isStop) {

            errorStackList[index] = stackList[index];
//            return;
        }

        stackList[index] = null;
        index--;
    }

    function stop() {

        isStop = true;
    }

    function getTrace() {

        Log.trace("---------- getTrace " + index + " ----------");

        var value = "";

        var errorIndex = errorStackList.length - 1;

        for (var i = errorIndex; i >= 0; i --) {

            var stack = errorStackList[i];
            var clazzName = stack.clazzName;
            var methodName = stack.methodName;
            value += "\t" + clazzName + "#" + methodName + "(";
            var args = stack.args;
            if (null != args) {

                for (var j = 0; j < args.length; j ++) {

                    value += args[j];
                    if (j != (args.length - 1)) {

                        value += ", ";
                    }
                }
            }
            value += ")\n";
        }

        return value;
    }

    function wrapping(clazz, clazzName) {

        for (var name in clazz) {

            wrapping2(clazz, clazzName, name);
        }
    }

    function wrapping2(clazz, clazzName, name) {

        var value = clazz[name];
        if ("function" == typeof value) {

            if (isTraceOnly) {

                clazz[name] = function () {

                    id++;
                    var currentId = id;

                    Log.trace("--- " + clazzName + "#" + name + " START(" + currentId + ") ---");

                    var rtn = value.apply(this, arguments);

                    Log.trace("--- " + clazzName + "#" + name + " END(" + currentId + ") ---");

                    return rtn;
                };

            } else {

                clazz[name] = function () {

                    id++;
                    var currentId = id;
                    var endStatus = "";

                    try{

                        start(clazzName, name, arguments);

                        Log.trace("--- " + clazzName + "#" + name + " START(" + currentId + ") ---");

                        var rtn = value.apply(this, arguments);

                        return rtn;

                    } catch(e) {

                        if (!isStop && e.stack) {
                            Log.error(e.stack);
                        }
                        endStatus = e.toString();
                        stop();
                        throw e;

                    } finally {

                        Log.trace("--- " + clazzName + "#" + name + " END(" + currentId + ") ---" + endStatus);

                        end();
                    }
                };

            }
        }
    }

    function insert(clazz, clazzName) {

        if (isStackTrace) {

            wrapping(clazz, clazzName);
            wrapping(clazz.prototype, clazzName);
        }
    }

    function errorHandler(message, url, line) {

        alert("エラーを検知しました。\n" +
              message + "\n" +
              url + ":" + line + "\n" +
              getTrace());
        return true;
    }

//    StackTrace.start = start;
//    StackTrace.end = end;
//    StackTrace.stop = stop;
    StackTrace.getTrace = getTrace;
    StackTrace.insert = insert;
    StackTrace.errorHandler = errorHandler;

})();


com.yscjp.jsapp.util.ArgChk = function () {};
(function() {

    var ArgChk = com.yscjp.jsapp.util.ArgChk;

    /**
     * argument が定義されているかを判定します.
     * 未定義の判定は、以下のコードにより行います。
     * <pre>if ("undefined" == typeof argument)</pre>
     * @param {Object} argument 定義済みかを判定するオブジェクト
     * @param {Object} defaultValue 未定義の場合に返すデフォルトの値
     * @return argument が定義済みの場合 argument をそのまま返します。
     * argument が未定義の場合、defaultValue が定義されていれば defaultValue を返します。
     * defaultValue も未定義の場合、JSNullPointerException をスローします。
     * @memberOf com.yscjp.jsapp.util.ArgChk
     */
    function notUndefined(argument, defaultValue) {
        if ("undefined" == typeof argument) {

            if ("undefined" != typeof defaultValue) {

                return defaultValue;
            }

            throw new Error("Argument is undefined.");
        }

        return argument;
    }

    function notUndefinedTypeOf(argument, type, defaultValue) {

        argument = notUndefined(argument, defaultValue);
        return typeOf(argument, type, defaultValue);
    }

    /**
     * argument が null 以外かを判定します.
     * argument が 未定義の場合も、null と判定されます。
     * @param {Object} argument null以外かを判定するオブジェクト
     * @param {Object} defaultValue nullの場合に返すデフォルトの値
     * @return argument がnull以外の場合 argument をそのまま返します。
     * argument がnullの場合、defaultValue が定義されていれば defaultValue を返します。
     * defaultValue も未定義の場合、JSNullPointerException をスローします。
     * @memberOf com.yscjp.jsapp.util.ArgChk
     */
    function notNull(argument, defaultValue) {
        if (null == argument) {

            if ("undefined" != typeof defaultValue) {

                return defaultValue;
            }

            throw new Error("Argument is null.");
        }

        return argument;
    }

    function notNullTypeOf(argument, type, defaultValue) {

        argument = notNull(argument, defaultValue);
        return typeOf(argument, type, defaultValue);
    }

    /**
     * argument が 空以外かを判定します.
     * 空の判定方法は以下のとおり。
     * <ul>
     * <li>argument が Object の時、null が空と判断されます。
     * <li>argument が数値の時、0 が空と判断されます。
     * <li>argument が文字列の時、空文字が空と判断されます。
     * </ul>
     * @param {Object} argument 空以外かを判定するオブジェクト
     * @param {Object} defaultValue 空の場合に返すデフォルトの値
     * @return argument が空以外の場合 argument をそのまま返します。
     * argument が空の場合、defaultValue が定義されていれば defaultValue を返します。
     * defaultValue も未定義の場合、JSIllegalArgumentException をスローします。
     * @memberOf com.yscjp.jsapp.util.ArgChk
     */
    function notEmpty(argument, defaultValue) {

        argument = notNull(argument, defaultValue);
        if (!argument) {

            if ("undefined" != typeof defaultValue) {

                return defaultValue;
            }

            throw new Error("Argument is illegal.");
        }

        return argument;
    }

    function notEmptyTypeOf(argument, type, defaultValue) {

        argument = notEmpty(argument, defaultValue);
        return typeOf(argument, type, defaultValue);
    }

    /**
     * argument が type かを判定します.
     * 空の判定方法は以下のとおり。
     * <pre>if (type != typeof argument)</pre>
     * 以下のコードは正常に終了し、argument を返します。
     * <ul>
     * <li>ArgChk.typeOf(0, "number");
     * <li>ArgChk.typeOf("9876", "string");
     * <li>ArgChk.typeOf(true, "boolean");
     * <li>ArgChk.typeOf(new Date(), "object");
     * </ul>
     * @param {Object} argument type かを判定するオブジェクト
     * @param {Object} defaultValue type 以外の場合に返すデフォルトの値
     * @return argument が空以外の場合 argument をそのまま返します。
     * argument が type 以外の場合、defaultValue が定義されていれば defaultValue を返します。
     * defaultValue も未定義の場合、JSIllegalArgumentException をスローします。
     * @memberOf com.yscjp.jsapp.util.ArgChk
     */
    function typeOf(argument, type, defaultValue) {

        notNull(type);
        argument = notNull(argument, defaultValue);

        if (type != typeof argument) {

            if ("undefined" != typeof defaultValue) {

                return defaultValue;
            }

            var message = "Argument is illegal type. type:" + type + " argument.type:" + typeof argument;
            throw new Error(message);
        }

        return argument;
    }

    /**
     * argument が instanceType で指定したクラスのインスタンスかを判定します.
     * 判定方法は以下のとおり。
     * <pre>if (!(argument instanceof instanceType))</pre>
     * 以下のコードは正常に終了し、argument を返します。
     * <ul>
     * <li>ArgChk.instanceOf(new Number(0), Number);
     * <li>ArgChk.instanceOf(new String("9876"), String);
     * <li>ArgChk.instanceOf(new Date(), Date);
     * <li>ArgChk.instanceOf(document.getElementById('target1'), Element);
     * <li>ArgChk.instanceOf(new Dummy2(), Dummy2)
     * </ul>
     * @param {Object} argument type かを判定するオブジェクト
     * @param {Object} defaultValue type 以外の場合に返すデフォルトの値
     * @return argument が空以外の場合 argument をそのまま返します。
     * argument が type 以外の場合、defaultValue が定義されていれば defaultValue を返します。
     * defaultValue も未定義の場合、JSIllegalArgumentException をスローします。
     * @memberOf com.yscjp.jsapp.util.ArgChk
     */
    function instanceOf(argument, instanceType, defaultValue) {

        notNull(instanceType);
        argument = notNull(argument, defaultValue);
        if (!(argument instanceof instanceType)) {

            if ("undefined" != typeof defaultValue) {

                return defaultValue;
            }

            throw new Error("Argument is illegal instanceType.");
        }

        return argument;
    }

    function instanceOfElement(element, defaultValue) {

        element = notNull(element, defaultValue);
        if (window.Element) {

            return instanceOf(element, window.Element, defaultValue);
        }

        /*
         * Element クラスが定義されていない。
         */
        ArgChk.notUndefined(element.nodeType);
        if (1 != element.nodeType) {

            throw new Error("Argument is not Element.:" + element.nodeType);
        }

        return element;
    }

    ArgChk.notUndefined = notUndefined;
    ArgChk.notNull = notNull;
    ArgChk.notEmpty = notEmpty;
    ArgChk.typeOf = typeOf;
    ArgChk.instanceOf = instanceOf;
    ArgChk.instanceOfElement = instanceOfElement;

})();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.util.ArgChk,
                                       "com.yscjp.jsapp.util.ArgChk");

com.yscjp.jsapp.util.JSString = function () {};
(function() {

    /**
     * string が prefix で始まっているかを判定します.
     * @return string が prefix で始まっている場合 true、それ以外の場合 false
     * @memberOf com.yscjp.jsapp.util.JSString
     */
    function startsWith(string, prefix) {

        if (string.length < prefix.length) {

            return false;
        }

        if (string.substring(0, prefix.length) != prefix) {

            return false;
        }

        return true;
    }

    /**
     * string が prefix で始まっているかを、大文字小文字を区別しないで判断します.
     * @return string が大文字小文字の区別無く prefix で始まっている場合 true、それ以外の場合 false
     * @memberOf com.yscjp.jsapp.util.JSString
     */
    function caseStartsWith(string, prefix) {

        return startsWith(string.toLowerCase(), prefix.toLowerCase());
    }

    /**
     * string が suffix で終わっているかを判定します.
     * @return string が suffix で終わっている場合 true、それ以外の場合 false
     * @memberOf com.yscjp.jsapp.util.JSString
     */
    function endsWith(string, suffix) {
        if (string.length < suffix.length) {

            return false;
        }

        if (string.substring(string.length - suffix.length) != suffix) {

            return false;
        }

        return true;
    }

    /**
     * string が suffix で終わっているかを、大文字小文字を区別しないで判断します.
     * @return string が大文字小文字の区別無く suffix で終わっている場合 true、それ以外の場合 false
     * @memberOf com.yscjp.jsapp.util.JSString
     */
    function caseEndsWith(string, suffix) {

        return endsWith(string.toLowerCase(), suffix.toLowerCase());
    }

    com.yscjp.jsapp.util.JSString.startsWith = startsWith;
    com.yscjp.jsapp.util.JSString.caseStartsWith = caseStartsWith;
    com.yscjp.jsapp.util.JSString.endsWith = endsWith;
    com.yscjp.jsapp.util.JSString.caseEndsWith = caseEndsWith;

})();


/**
 * XML関連のユーティリティ
 * XML関連は、Webブラウザ毎の違いが大きいので、ここで吸収する。
 */
com.yscjp.jsapp.util.XmlUtil = function () {};
(function() {

    var XmlUtil = com.yscjp.jsapp.util.XmlUtil;


    /*
     * 文字列のパース
     * @param stringValue XMLドキュメントにパースする、文字列。
     * @return パースした結果の、XMLドキュメント
     */
    XmlUtil.perseFromString = function (stringValue) {
        if (window.DOMParser) {
            var parser = new DOMParser();
            return parser.parseFromString(stringValue, 'text/xml');
        } else {
            var newDoc = new ActiveXObject("Microsoft.XMLDOM");
            newDoc.setProperty("SelectionLanguage", "XPath");
            newDoc.loadXML(stringValue);
            return newDoc;
        }
    };

    /*
     * XMLドキュメントのシリアライズ(テキスト化)
     * @param node テキストに変換するXMLノード
     * @return シリアライズした文字列
     */
    XmlUtil.writeToString = function (node) {
        if (window.XMLSerializer) {
            var serializer = new XMLSerializer();
            return serializer.serializeToString(node);
        } else {
            return node.xml;
        }
    };

})();


/**
 * XMLHttpRequest関連のユーティリティ
 * XMLHttpRequest関連は、Webブラウザ毎の違いが大きいので、ここで吸収する。
 * クロスドメインで通信することを前提としています。
 */
com.yscjp.jsapp.util.Request = function () {};
(function() {

    var Log = com.yscjp.jsapp.util.Log;
    var ArgChk = com.yscjp.jsapp.util.ArgChk;
    var Request = com.yscjp.jsapp.util.Request;

    var __PFunc = function () {};

    __PFunc.getRequest = function (successHandler, errorHandler, abortHandler, timeoutHandler, timeout) {

        var request = null;

        if (window.XDomainRequest) {

            Log.debug("window.XDomainRequest");

            /*
             * IE8以降
             */
            request = new XDomainRequest();
            request.onload = successHandler;
            request.onerror = function () { errorHandler("onerror"); };
            request.onabort = function () { abortHandler("onabort"); };
            request.ontimeout = function () { timeoutHandler("ontimeout"); };
            request.timeout = timeout;

            return request;

        } else if (window.XMLHttpRequest) {

            Log.debug("window.XMLHttpRequest");

            /*
             * 標準とIE7
             */
            request = new XMLHttpRequest();

        } else if (window.ActiveXObject) {

            Log.debug("window.ActiveXObject");

            /*
             * IE6
             */
            request = new ActiveXObject("Msxml2.XMLHTTP");
        }

        if ("undefined" == typeof request.onload) {

            /*
             * Level 1
             */
            request.onreadystatechange = function() {

                if (4 == request.readyState) {

                    if (200 == request.status) {

                        successHandler();

                    } else {

                        errorHandler(request.status);
                    }
                }
            };

        } else {

            /*
             * Level 2
             */
            request.onload = function() {

                if (200 == request.status) {

                    successHandler();

                } else {

                    errorHandler("status:" + request.status);
                }
            };

            request.onerror = function () { errorHandler("onerror"); };
            request.onabort = function () { abortHandler("onabort"); };
            request.ontimeout = function () { timeoutHandler("ontimeout"); };
//            request.timeout = timeout;
        }

        return request;
    };

    __PFunc.asyncRequest = function (method, url, contentType, message, successHandler, errorHandler, abortHandler, timeoutHandler, timeout) {

        ArgChk.notUndefined(method);
        ArgChk.notUndefined(url);
        ArgChk.notUndefined(successHandler);
        errorHandler = ArgChk.notUndefined(errorHandler, function() {});
        abortHandler = ArgChk.notUndefined(abortHandler, errorHandler);
        timeoutHandler = ArgChk.notUndefined(timeoutHandler, errorHandler);
        timeout = ArgChk.notUndefined(timeout, 60000);

        var request = __PFunc.getRequest(successHandler, errorHandler, abortHandler, timeoutHandler, timeout);

        request.open(method, url, true);

        if (timeout && "undefined" != typeof request.timeout && !window.XDomainRequest) {

            request.timeout = timeout;
        }

        if (contentType && "undefined" != typeof request.setRequestHeader) {

            request.setRequestHeader("Content-Type" , contentType);
        }

        request.send(message);
    };

    /**
     * 同期で通信を行う。
     */
    Request.syncRequest = function (url) {

        var handler = function() {};
        var request = __PFunc.getRequest(handler, handler, handler, handler, 0);
        request.open("GET", url, false);
        request.send();
    };

    /**
     * GETで通信を行う。
     */
    Request.get = function (url, successHandler, errorHandler, abortHandler, timeoutHandler, timeout) {

        __PFunc.asyncRequest("GET", url, null, null, successHandler, errorHandler, abortHandler, timeoutHandler, timeout);
    };

    /**
     * POSTで通信を行う。
     */
    Request.post = function (url, contentType, message, successHandler, errorHandler, abortHandler, timeoutHandler, timeout) {

        __PFunc.asyncRequest("POST", url, contentType, message, successHandler, errorHandler, abortHandler, timeoutHandler, timeout);
    };

    com.yscjp.jsapp.util.StackTrace.insert(__PFunc, "com.yscjp.jsapp.util.Request.__PFunc");

 })();

com.yscjp.jsapp.util.StackTrace.insert(com.yscjp.jsapp.util.Request, "com.yscjp.jsapp.util.Request");


com.yscjp.jsapp.util.DomUtil = function () {};

(function () {

   var DomUtil = com.yscjp.jsapp.util.DomUtil;

   DomUtil.ELEMENT_NODE= document.ELEMENT_NODE;                                // 1
   if (! DomUtil.ELEMENT_NODE) {
       DomUtil.ELEMENT_NODE = 1;
   }
   DomUtil.ATTRIBUTE_NODE= document.ATTRIBUTE_NODE;                            // 2
   if (! DomUtil.ATTRIBUTE_NODE) {
       DomUtil.ATTRIBUTE_NODE = 2;
   }
   DomUtil.TEXT_NODE= document.TEXT_NODE;                                      // 3
   if (! DomUtil.TEXT_NODE) {
       DomUtil.TEXT_NODE = 3;
   }
   DomUtil.CDATA_SECTION_NODE= document.CDATA_SECTION_NODE;                    // 4
   if (! DomUtil.CDATA_SECTION_NODE) {
       DomUtil.CDATA_SECTION_NODE = 4;
   }
   DomUtil.ENTITY_REFERENCE_NODE= document.ENTITY_REFERENCE_NODE;              // 5
   if (! DomUtil.ENTITY_REFERENCE_NODE) {
       DomUtil.ENTITY_REFERENCE_NODE = 5;
   }
   DomUtil.ENTITY_NODE= document.ENTITY_NODE;                                  // 6
   if (! DomUtil.ENTITY_NODE) {
       DomUtil.ENTITY_NODE = 6;
   }
   DomUtil.PROCESSING_INSTRUCTION_NODE= document.PROCESSING_INSTRUCTION_NODE;  // 7
   if (! DomUtil.PROCESSING_INSTRUCTION_NODE) {
       DomUtil.PROCESSING_INSTRUCTION_NODE = 7;
   }
   DomUtil.COMMENT_NODE= document.COMMENT_NODE;                                // 8
   if (! DomUtil.COMMENT_NODE) {
       DomUtil.COMMENT_NODE = 8;
   }
   DomUtil.DOCUMENT_NODE= document.DOCUMENT_NODE;                              // 9
   if (! DomUtil.DOCUMENT_NODE) {
       DomUtil.DOCUMENT_NODE = 9;
   }
   DomUtil.DOCUMENT_TYPE_NODE= document.DOCUMENT_TYPE_NODE;                    // 10
   if (! DomUtil.DOCUMENT_TYPE_NODE) {
       DomUtil.DOCUMENT_TYPE_NODE = 10;
   }
   DomUtil.DOCUMENT_FRAGMENT_NODE= document.DOCUMENT_FRAGMENT_NODE;            // 11
   if (! DomUtil.DOCUMENT_FRAGMENT_NODE) {
       DomUtil.DOCUMENT_FRAGMENT_NODE = 11;
   }
   DomUtil.NOTATION_NODE= document.NOTATION_NODE;                              // 12
   if (! DomUtil.NOTATION_NODE) {
       DomUtil.NOTATION_NODE = 12;
   }

   /*
    * ターゲットノードの子供をすべて削除する。
    */
   DomUtil.removeAllChild = function (targetNode) {

       while (targetNode.hasChildNodes()) {

           targetNode.removeChild(targetNode.firstChild);
       }
   };


   /*
    * targetNodeの子孫から、指定のidを持つElementを取得する。
    */
   DomUtil.getElementByIdFromNode = function (targetNode, targetId) {

       /*
        * ターゲットが子供を持たない場合は、処理終了。
        */
       if (targetNode.hasChildNodes()) {

           var childList = targetNode.childNodes;
           var count = childList.length;

           for (var i = 0; i < count; i ++) {

               var elementItem = childList.item(i);

               if (elementItem.nodeType == DomUtil.ELEMENT_NODE) {

                   /*
                    * エレメントノードの場合、idをチェックし、
                    * 該当しなければ、リカーシブ
                    */
                   if (elementItem.id == targetId) {
                       return elementItem;
                   }
                   var rtnElement =
                       DomUtil.getElementByIdFromNode(elementItem, targetId);

                   if (rtnElement) {

                       return rtnElement;
                   }
               }
           }
       }
       return null;
   };


   /**
    * 指定要素の子孫要素から、指定の属性が指定の値となっている要素を取得する。
    * 該当する要素が複数存在する場合でも、最初に発見した要素を返す。
    * 該当する要素が、指定要素の子孫に存在しない場合、nullを返す。
    * @param 要素 : 指定要素。この子孫の中から、該当する要素を検索する。
    * @param 文字列 : 指定属性。
    * @param 文字列 : 値
    * @return 要素 : 発見した要素。該当する要素が存在しない場合、null。
    */
   DomUtil.getElementByAttributeValue =
       function (targetNode, attributeKey, attributeValue) {

       if ("class" == attributeKey) {

           attributeKey = "className";
       }

       /*
        * ターゲットが子供を持たない場合は、処理終了。
        */
       if (targetNode.hasChildNodes()) {

           var childList = targetNode.childNodes;
           var count = childList.length;
           for (var i = 0; i < count; i ++) {

               var elementItem = childList.item(i);
               if (elementItem.nodeType == DomUtil.ELEMENT_NODE) {

                   /*
                    * エレメントノードの場合、属性値をチェックし、
                    * 該当しなければ、リカーシブ
                    * 属性周りは、DOM Level 1程度のものも、IE5.5は
                    * まともにサポートしていないので、早く捨てたい。
                    */
                   //var value = elementItem.getAttribute(attributeKey);
                   var value = elementItem[attributeKey];
                   if (value) {

                       var valueList = value.split(" ");

                       for (var j = 0; j < valueList.length; j ++) {

                           if (valueList[j] == attributeValue) {

                               return elementItem;
                           }
                       }
                   }

                   var rtnElement =
                       DomUtil.getElementByAttributeValue(elementItem,
                               attributeKey,
                               attributeValue);
                   if (rtnElement) {

                       return rtnElement;
                   }
               }
           }
       }
       return null;
   };


    /*
     * ドキュメントとidを指定して、該当するノードの先頭にある
     * テキストノードから、テキストを取得する。
     * ドキュメントが指定されない場合は、documentを使用する。
     */
    DomUtil.getTextValue = function (targetId, targetDoc) {

        if (null == targetDoc) {

            targetDoc = document;
        }

        var targetElement = targetDoc.getElementById(targetId);
        if (targetElement == null) {
            throw 'DomUtil@getTextValue:' +
                'no element applicable to a targerId.' +
                ' targetId('+targetId+')';
        }
        return DomUtil.getTextValueByNode(targetElement);
    };


     /*
      * 指定エレメントの先頭にあるテキストノードから、
      * テキストを取得する。
      */
     DomUtil.getTextValueByNode = function (targetNode) {

         if (targetNode.hasChildNodes()) {

             /*
              * 子ノードがある場合、先頭の子ノードがテキストノードかをチェックし、
              * テキストノードの場合は、そのvalueを返す。
              */
             if (targetNode.firstChild.nodeType == DomUtil.TEXT_NODE) {

                 return targetNode.firstChild.nodeValue;
             }
         }

         /*
          * 子ノードを持たない場合や、先頭の子ノードがテキストノード以外の場合は、
          * サイズ０の文字列を返す。
          */
         return '';
     };


     /*
      * 指定のエレメントに、テキストノード追加し、指定のテキストをセットする。
      * targetNode : element
      * textValue  : String
      * viewDoc    : document : nullの場合は、documentを使用する。
      */
     DomUtil.setTextValue = function (targetElement, textValue, viewDoc) {

         if (null == viewDoc) {

             viewDoc = targetElement.ownerDocument;
         }

         if (targetElement.nodeType != DomUtil.ELEMENT_NODE) {

             com.yscjp.jsapp.util.Log.error("DomUtil.setTextValue:No ELEMENT_NODE");
             return;
         }

         DomUtil.removeAllChild(targetElement);
         var textValueNode = viewDoc.createTextNode(textValue);
         targetElement.appendChild(textValueNode);
     };


     /*
      * ディープクローン
      * valueTag配下のxml(html)の構造を、targetTag配下にコピーする。
      * targetTag コピー先のタグ
      * valueTag コピー元となるタグ
      */
     DomUtil.deepClone = function (targetTag, valueTag) {

         /*
          * コピー元に子ノードが存在する場合のみ、処理を行う。
          */
         if (valueTag.hasChildNodes()) {

             var targetDoc = targetTag.ownerDocument;
             if (! targetDoc) {

                 targetDoc = document;
             }

             for (var i = 0; i < valueTag.childNodes.length; i ++) {

                 var valueChild = valueTag.childNodes.item(i);

                 if (valueChild.nodeType == DomUtil.ELEMENT_NODE) {

                     /**
                      * エレメントノードの場合は、タグ名でノードを作成し、
                      * アトリビュートをコピーする。
                      * その後、まだ子が有る場合は、リカーシブ。
                      */
                     var targetEleNode = targetDoc.createElement(valueChild.tagName);
                     targetEleNode = targetTag.appendChild(targetEleNode);
                     var valueAtts = valueChild.attributes;
                     targetEleNode.className = valueChild.className;

                     //        for (var j = 0; j < valueAtts.length; j ++) {

                     //          var attNode = valueAtts.item(j);
                     /**
                      * 本来のDOMの仕様であれば、下のコメントとなっているコードで、
                      * 正しく動作するはずだが、IE5.5がサポートしていないので、
                      * やむなく、変更する。
                      */
                     //          targetEleNode.setAttribute(attNode.name, attNode.value);
                     //          targetAtts.setNamedItem(attNode);
                     //        }

                     /*
                      * この次の処理は、イメージの読み込みを正しく行わない
                      * Webブラウザが存在するために、しょうがなく行う。
                      */
                     if (valueChild.tagName == 'IMG') {

                         var srcurl = valueChild.getAttribute('src');
                         targetEleNode.src = srcurl;
                     }
                     DomUtil.deepClone(targetEleNode, valueChild);

                 } else if (valueChild.nodeType == DomUtil.TEXT_NODE) {

                     /**
                      * テキストノードの場合は、テキストでノードを作成し、
                      * 子として追加する。
                      */
                     var textNode = targetDoc.createTextNode(valueChild.nodeValue);
                     targetTag.appendChild(textNode);
                 }
             }
         }
     };


     /*
      * ディープクローン
      * sourceNode配下のxml(html)の構造を、targetNode配下にコピーする。
      * コピーの対象となるのは、エレメントノードとテキストノードのみで、
      * コメントなどは、コピーされない。
      * エレメントノードの属性は、デフォルトの状態では、
      * 設定されている全ての属性をコピーするが、
      * DomUtil.cloneAttributes(nodeTo, nodeFrom, isUnImp)
      * メソッドを適当なメソッドで上書きすることにより、
      * 指定の属性のみコピーするように、変更することが出来る。
      *
      * targetNode コピー先のタグ
      * sourceNode コピー元となるタグ
      * isUnImp 属性を全てコピーするかを指定するフラグ
      *         null以外が指定された場合、全ての属性をコピーする。
      */
     DomUtil.deepClone2 = function (targetNode, sourceNode, isUnImp) {

         /*
          * コピー元に子ノードが存在する場合のみ、処理を行う。
          */
         if (! sourceNode.hasChildNodes()) {

             return;
         }

         var targetDocument = targetNode.ownerDocument;

         if (! targetDocument) {

             targetDocument = document;
         }

         for (var i = 0; i < sourceNode.childNodes.length; i ++) {

             var sourceChild = sourceNode.childNodes.item(i);

             if (sourceChild.nodeType == DomUtil.ELEMENT_NODE) {

                 /*
                  * エレメントノードの場合は、タグ名でノードを作成し、
                  * アトリビュートをコピーする。
                  * その後、まだ子が有る場合は、リカーシブ。
                  */
                 var targetEleNode = targetDocument.createElement(sourceChild.tagName);
                 targetEleNode = targetNode.appendChild(targetEleNode);
                 DomUtil.cloneAttributes(targetEleNode, sourceChild, isUnImp);
                 //            targetEleNode.className = sourceChild.className;

                 DomUtil.deepClone2(targetEleNode, sourceChild, isUnImp);

             } else if (sourceChild.nodeType == DomUtil.TEXT_NODE) {

                 /**
                  * テキストノードの場合は、テキストでノードを作成し、
                  * 子として追加する。
                  */
                 var textNode = targetDocument.createTextNode(sourceChild.nodeValue);
                 targetNode.appendChild(textNode);
             }
         }
     };


     DomUtil.cloneAttributes = function (nodeTo, nodeFrom, isUnImp) {

         var fromAttributes = nodeFrom.attributes;

         for (var j = 0; j < fromAttributes.length; j ++) {

             var fromAttribute = fromAttributes.item(j);

             /**
              * 本来のDOMの仕様であれば、下のコメントとなっているコードで、
              * 正しく動作するはずだが、IE5.5がサポートしていないので、
              * やむなく、変更する。
              */
             nodeTo.setAttribute(fromAttribute.name, fromAttribute.value);
             //        targetAtts.setNamedItem(fromAttribute);   // IE5.5 not supported
         }
     };

})();

com.yscjp.jsapp.util.CSSView = function () {};
(function() {

    var JSString = com.yscjp.jsapp.util.JSString;
    var CSSView = com.yscjp.jsapp.util.CSSView;

    var sizeTbl = new Object();
    var testDiv = null;

    CSSView.getElementStylePixcel = function (element, property) {

        var cssUnit = CSSView.getElementStyle(element, property);
        return CSSView.toPixcel(cssUnit);
    };

    /**
     * 要素に指定されているスタイルを取得する。
     */
    CSSView.getElementStyle = function (element, property) {

        if (element.currentStyle) {

            return element.currentStyle[property];

        } else if (document.defaultView.getComputedStyle) {

            var cssDeclaration = document.defaultView.getComputedStyle(element, '');
            return cssDeclaration[property];

        } else {

            throw "getElementStyle not supported.";
        }
    };

    /*
     * CSSの単位で渡された値を、ピクセル単位に変換して返します。
     * 現時点では、pxとemのみをサポートします。
     * emを指定された場合、値は近似値となります。
     * （pxのみ実装されています）
     */
    CSSView.toPixcel = function (cssUnit) {

        if (null == cssUnit || 0 == cssUnit.length) {

            return 0;
        }

        if ("undefined" != typeof sizeTbl[cssUnit]) {

            return sizeTbl[cssUnit];
        }

        var returnNumber = 0;
        if (JSString.caseEndsWith(cssUnit, "px")) {

            returnNumber = cssUnit.replace("px", "") - 0;
            if (isNaN(returnNumber)) {

                /*
                 * IEの場合、正しい仕様通りの結果など望むべくもない。
                 */
                return null;
            }
        } else if (JSString.caseEndsWith(cssUnit, "in") ||
                   JSString.caseEndsWith(cssUnit, "cm") ||
                   JSString.caseEndsWith(cssUnit, "mm") ||
                   JSString.caseEndsWith(cssUnit, "pt") ||
                   JSString.caseEndsWith(cssUnit, "pc") ||
                   JSString.caseEndsWith(cssUnit, "em") ||
                   JSString.caseEndsWith(cssUnit, "ex")) {

            if (!testDiv) {

                testDiv = document.createElement("div");
                testDiv = document.body.appendChild(testDiv);
                testDiv.style.padding = '0';
                testDiv.style.border = 'none';
                if (com.yscjp.jsapp.util.UA.isFixedSupport()) {

                    testDiv.style.position = 'fixed';

                } else {

                    testDiv.style.position = 'absolute';
                }
                testDiv.style.top = '-100em';
                testDiv.style.left = '-100em';
            }

            testDiv.style.width = cssUnit;
            returnNumber = testDiv.offsetWidth;
        } else {

            /*
             * auto 等、変換不能な値が指定されている。
             */
            return null;
        }

        sizeTbl[cssUnit] = returnNumber;

        return returnNumber;
    };

    CSSView.getStyleTop = function (element) {
        var cssVal = CSSView.getElementStyle(element, "top");
        var pxVal = CSSView.toPixcel(cssVal);
        return pxVal;
    };

    CSSView.getStyleLeft = function (element) {
        var cssVal = CSSView.getElementStyle(element, "left");
        var pxVal = CSSView.toPixcel(cssVal);
        return pxVal;
    };

    CSSView.getStyleHeight = function (element) {
        var cssVal = CSSView.getElementStyle(element, "height");
        var pxVal = CSSView.toPixcel(cssVal);
        return pxVal;
    };

    CSSView.getStyleWidth = function (element) {
        var cssVal = CSSView.getElementStyle(element, "width");
        var pxVal = CSSView.toPixcel(cssVal);
        return pxVal;
    };

})();

if (!com.yscjp.jsapp.util.html) com.yscjp.jsapp.util.html = {};

/**
 * HTMLのコントロールを操作するための、ユーティリティクラスです.
 * @class コンソールへログの出力を行います.
 */
com.yscjp.jsapp.util.html.Control = function () {};
(function() {
    var Control = com.yscjp.jsapp.util.html.Control;
    Control.setValuesByName = function (name, values) {
    };
})();


/*
 * HTMLユーティリティ
 * JavascriptでHTMLを操作する際に、便利なユーティリティと定数定義
 * ある程度のWebブラウザ間の違いは、この中で吸収する。
 *
 * このソース内で定義されている変数とメソッドは、全てクラス変数・
 * クラスメソッドです。オブジェクトのインスタンスを作成する必要はありません。
 *
 * Version 0.0.0.1
 */

com.yscjp.jsapp.util.JSHtmlUtil = function () {};
(function() {

    var Log = com.yscjp.jsapp.util.Log;
    var CSSView = com.yscjp.jsapp.util.CSSView;
    var JSHtmlUtil = com.yscjp.jsapp.util.JSHtmlUtil;

    /*
     * 指定要素に、指定のクラスを追加する。
     */
    JSHtmlUtil.addClassValue = function (element, classValue) {

        if (null == classValue || 0 == classValue.length) {

            /*
             * nullや空文字の場合は何もしない。
             */
            return;
        }

        var ptn = new RegExp("\\b"+classValue+"\\b");

        /*
         * 既に存在すれば、なにもしない。
         */
        if (-1 == element.className.search(ptn)) {

            element.className += " " + classValue;
        }
    };

    /*
     * 指定要素から、指定のクラスを削除する。
     */
    JSHtmlUtil.removeClassValue = function (element, classValue) {

        return com.yscjp.jsapp.util.JSHtmlUtil.replaceClassValue(element, classValue, "");
    };

    /*
     * 指定要素のクラスを、oldValueからnewValueに変更する。
     */
    JSHtmlUtil.replaceClassValue = function (element, oldValue, newValue) {

        if (null == oldValue || 0 == oldValue.length) {

            /*
             * nullや空文字の場合は何もしない。
             */
            return;
        }

        var ptn = new RegExp("\\b"+oldValue+"\\b", "g");
        var className = element.className;
        element.className = className.replace(ptn, newValue);

        if (element.className == className) {

            /*
             * 変更なし
             */
            return false;
        }

        return true;
    };

    /*
     * 指定要素のクラスに、valueが存在するかチェックします。
     */
    JSHtmlUtil.matchValue = function (element, value) {

        if (!value) {

            return false;
        }

        var ptn = new RegExp("\\b" + value + "\\b", "g");
        return null != element.className.match(ptn);
    };


    /**
     * 指定要素までのスクロール
     * @param target : 要素 : スクロールさせたい要素。
     *                 この要素の上辺までスクロールさせる。
     * @param margin : 数値(px) : スクロール位置を、targetの上辺 - marginとする。
     */
    JSHtmlUtil.scrollTo = function (target, margin) {

        if (null == margin) {

            margin = 0;
        }

        var top = target.offsetTop - margin;
        window.scrollTo(0, top);
    };


    /**
     * 指定要素の上端の位置を、ピクセル単位で返します。
     * @param target : 要素 : 上端を取得する要素
     */
    JSHtmlUtil.getTopPixel = function (target) {

        var allPix = target.offsetTop;
        var offsetParent = target.offsetParent;
        while (null != offsetParent) {

            allPix += offsetParent.offsetTop;
            offsetParent = offsetParent.offsetParent;
        }

//        return allPix + CSSView.toPixcel(CSSView.getElementStyle(target, "borderTopWidth"));
        return allPix;
    };

    /**
     * 指定要素の左端の位置を、ピクセル単位で返します。
     * @param target : 要素 : 左端を取得する要素
     */
    JSHtmlUtil.getLeftPixel = function (target) {

        var allPix = target.offsetLeft;
        var offsetParent = target.offsetParent;
        while (null != offsetParent) {

            allPix += offsetParent.offsetLeft;
            offsetParent = offsetParent.offsetParent;
        }

        //return allPix + CSSView.toPixcel(CSSView.getElementStyle(target, "borderLeftWidth"));
        return allPix;
    };

    /**
     * 指定要素の高さを、ピクセル単位で返します。
     * @param target : 要素 : 高さを取得する要素
     */
    JSHtmlUtil.getHeightPixel = function (target) {

        return target.clientHeight;
    };

    /**
     * 指定要素の幅を、ピクセル単位で返します。
     * @param target : 要素 : 幅を取得する要素
     */
    JSHtmlUtil.getWidthPixel = function (target) {

        return target.clientWidth;
    };

    /**
     * 指定のselect要素に、option要素を追加します。
     * option要素のvalueには引数のvalueを、表示内容にはdisplayをそれぞれ設定します。
     * @param select : 要素 : optionを追加するselect要素
     * @param value : 文字列 : option要素のvalueに設定する値
     * @param display : 文字列 : option要素の表示内容とする値
     */
    JSHtmlUtil.addOption = function (select, value, display) {

        var ownerDocument;
        if (select.ownerDocument) {

            ownerDocument = select.ownerDocument;

        } else {

            ownerDocument = select.document;
        }
        var option = ownerDocument.createElement("option");
        option.value = value;
        option.text = display;
        try {

            select.add(option, null);

        } catch (e) {

            select.add(option);
        }
    };

    /**
     * 指定のselect要素の、全てのoption要素を削除します。
     * @param select : 要素 : optionを削除するselect要素
     */
    JSHtmlUtil.removeAllOption = function (select) {

        var length = select.length;
        for (var i = 0; i < length; i++) {

            select.remove(0);
        }
    };

    /**
     * 指定のテーブルに行を追加します。
     * cellCount が指定されている場合、指定された数のセルを行に追加します。
     * @param table : 要素 : 行を追加するテーブル要素
     * @param cellCount : 数値 : 行に追加するセルの数（オプション）
     * @return 追加した行
     */
    JSHtmlUtil.addRow = function (table, cellCount) {

        if ("undefined" == typeof cellCount) {

            cellCount = 0;
        }

        var row = table.insertRow(-1);
        for (var i = 0; i < cellCount; i++) {

            row.insertCell(-1);
        }

        return row;
    };

    JSHtmlUtil.removeAllRows = function (table) {

        while (0 != table.rows.length) {

            table.deleteRow(-1);
        }
    };

})();

