package com.yscjp.webtools.paprika;

import java.awt.Image;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.TrayIcon.MessageType;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import javax.imageio.ImageIO;

import org.apache.catalina.Context;
import org.apache.catalina.LifecycleException;
import org.apache.catalina.Wrapper;
import org.apache.catalina.startup.Tomcat;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.yscjp.webtools.gateway.servlet.GatewayServlet;
import com.yscjp.webtools.robot.servlet.KeyPressServlet;
import com.yscjp.webtools.robot.servlet.KeyReleaseServlet;
import com.yscjp.webtools.robot.servlet.KeyTypeServlet;
import com.yscjp.webtools.robot.servlet.MouseClickServlet;
import com.yscjp.webtools.robot.servlet.MouseMoveServlet;
import com.yscjp.webtools.robot.servlet.MousePressServlet;
import com.yscjp.webtools.robot.servlet.MouseReleaseServlet;
import com.yscjp.webtools.robot.servlet.MouseWheelServlet;
import com.yscjp.webtools.robot.servlet.ScreenCaptureServlet;
import com.yscjp.webtools.robot.servlet.TypeTextServlet;
import com.yscjp.webtools.share.servlet.AutoregisterServletBase;


public class Server {

    static {

        System.setProperty("org.apache.commons.logging.Log",
                           "org.apache.commons.logging.impl.SimpleLog");
    }

    private static Log LOG = LogFactory.getLog(Server.class);
    private int port = -1;

    public Server() {}


    private void init() throws Exception {

        Image image = ImageIO.read(getClass().getResourceAsStream("y.png"));
        final TrayIcon icon = new TrayIcon(image);
        icon.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                icon.displayMessage("サーバのポート",
                    "サーバは、ポート（" + port + "）で実行中です。",
                    MessageType.WARNING);
            }
        });

        PopupMenu menu = new PopupMenu();
        MenuItem exitItem = new MenuItem("終了");
        exitItem.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {

                System.exit(0);
            }
        });
        menu.add(exitItem);
        icon.setPopupMenu(menu);

        SystemTray.getSystemTray().add(icon);
    }


    private void run() {

        Tomcat tomcat = new Tomcat();

        File f = new File("./");
        String wwwRoot = f.getAbsolutePath();
        LOG.info("WWW Root:" + wwwRoot);
        Context wwwCtx = tomcat.addContext("/", wwwRoot);
        Wrapper servlet = Tomcat.addServlet(wwwCtx, "default", "org.apache.catalina.servlets.DefaultServlet");
        servlet.setLoadOnStartup(1);
        servlet.setOverridable(true);

        wwwCtx.addServletMapping("/", "default");
        wwwCtx.addWelcomeFile("index.html");
        wwwCtx.addWelcomeFile("index.htm");

        InputStream inputStream = getClass().getResourceAsStream("mime.types");
        BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream));
        while (true) {

            try {

                String line = reader.readLine();
                if (null == line) {
                    break;
                }

                String[] mimeInfo = line.split(",");
                if (2 != mimeInfo.length) {

                    LOG.warn(line);
                    continue;
                }

                wwwCtx.addMimeMapping(mimeInfo[0], mimeInfo[1]);
                LOG.debug("\"" + mimeInfo[0] + "\"\t:\"" + mimeInfo[1] + "\"");

            } catch (IOException e) {

                LOG.error("", e);
                System.exit(0);
            }
        }

        File tmpDir = new File(System.getProperty("java.io.tmpdir"));
        Context robotCtx = tomcat.addContext("/__robot__", tmpDir.getAbsolutePath());

        registration(robotCtx, new MouseMoveServlet());
        registration(robotCtx, new MousePressServlet());
        registration(robotCtx, new MouseReleaseServlet());
        registration(robotCtx, new MouseClickServlet());
        registration(robotCtx, new MouseWheelServlet());
        registration(robotCtx, new KeyPressServlet());
        registration(robotCtx, new KeyReleaseServlet());
        registration(robotCtx, new KeyTypeServlet());
        registration(robotCtx, new TypeTextServlet());
        registration(robotCtx, new ScreenCaptureServlet());

        Context gatewayCtx = tomcat.addContext("/__gateway__", tmpDir.getAbsolutePath());

        registration(gatewayCtx, new GatewayServlet());

        int[] PORTS = new int[] { 12100, 12200, 12400, 12500, 12600 };
        for (int i : PORTS) {
            port = i;
            tomcat.setPort(port);
            try {

                tomcat.start();
                break;

            } catch (LifecycleException e) {}
        }

        tomcat.getConnector().setUseBodyEncodingForURI(true);

        LOG.info("---------- 開始 ---------- port:" + port);

        tomcat.getServer().await();

    }


    private void registration(Context ctx, AutoregisterServletBase autoregisterServletBase) {

        Tomcat.addServlet(ctx, autoregisterServletBase.getRegistrationName(), autoregisterServletBase);
        ctx.addServletMapping(autoregisterServletBase.getRegistrationURL(), autoregisterServletBase.getRegistrationName());
    }


    public static void main(String[] args) throws Exception {

        Server server = new Server();
        server.init();
        server.run();
    }
}
