package com.yscjp.webtools.robot;

import java.awt.AWTException;
import java.awt.GraphicsDevice;
import java.awt.MouseInfo;
import java.awt.Point;
import java.awt.PointerInfo;
import java.awt.Rectangle;
import java.awt.Robot;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;

import javax.imageio.ImageIO;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

public class RobotUtilities {

    private static final int DIVIDE_PX = 2;
    private static final int DIVIDE_WAIT = 2;

    private static Log LOG = LogFactory.getLog(RobotUtilities.class);

    private static Timer mouseReleaseTimer = null;

    private static Map<Integer, Integer> convertKeycodeMap = new HashMap<Integer, Integer>();
    static {

        /*
         * JavaScript と Java のキーコードの違いを変換する。
         */
        // JavaScript Java
        convertKeycodeMap.put(220, 92); // VK_BACK_SLASH 220 92
        convertKeycodeMap.put(226, 92); // VK_BACK_SLASH 220 92
        convertKeycodeMap.put(221, 93); // VK_CLOSE_BRACKET 221 93
        convertKeycodeMap.put(222, 514); // VK_CIRCUMFLEX 222 514
        convertKeycodeMap.put(186, 513); // VK_COLON 186 513
        convertKeycodeMap.put(187, 59); // VK_SEMICOLON 187 59
        convertKeycodeMap.put(188, 44); // VK_COMMA 188 44
        convertKeycodeMap.put(189, 45); // VK_MINUS 189 45
        // convertKeycodeMap.put( 93, 525); // VK_CONTEXT_MENU 93 525
        // convertKeycodeMap.put( 46, 127); // VK_DELETE 46 127
        convertKeycodeMap.put(13, 10); // VK_ENTER 14 10
        // convertKeycodeMap.put( 124, 61440); // VK_F13 124 61440
        // convertKeycodeMap.put( 125, 61441); // VK_F14 125 61441
        // convertKeycodeMap.put( 126, 61442); // VK_F15 126 61442
        // convertKeycodeMap.put( 127, 61443); // VK_F16 127 61443
        // convertKeycodeMap.put( 128, 61444); // VK_F17 128 61444
        // convertKeycodeMap.put( 129, 61445); // VK_F18 129 61445
        // convertKeycodeMap.put( 130, 61446); // VK_F19 130 61446
        // convertKeycodeMap.put( 131, 61447); // VK_F20 131 61447
        // convertKeycodeMap.put( 132, 61448); // VK_F21 132 61448
        // convertKeycodeMap.put( 133, 61449); // VK_F22 133 61449
        // convertKeycodeMap.put( 134, 61450); // VK_F23 134 61450
        // convertKeycodeMap.put( 135, 61451); // VK_F24 135 61451
        // convertKeycodeMap.put( 6, 156); // VK_HELP 6 156
        // convertKeycodeMap.put( 45, 155); // VK_INSERT 45 155
        // convertKeycodeMap.put( 224, 157); // VK_META 224 157
        convertKeycodeMap.put(219, 91); // VK_OPEN_BRACKET 219 91
        convertKeycodeMap.put(190, 46); // VK_PERIOD 190 46
        convertKeycodeMap.put(44, 154); // VK_PRINTSCREEN 44 154
        convertKeycodeMap.put(191, 47); // VK_SLASH 191 47
        convertKeycodeMap.put(192, 512); // VK_AT (@) 192 512
    }


    private static Robot getRobot() {

        PointerInfo pointerInfo = MouseInfo.getPointerInfo();
        GraphicsDevice graphicsDevice = pointerInfo.getDevice();

        try {

            return new Robot(graphicsDevice);

        } catch (AWTException e) {

            throw new RobotException(e);
        }
    }


    public static synchronized void mouseMove(int x, int y) {

        mouseMove(getRobot(), x, y);
    }


    private static void mouseMove(Robot robot, int x, int y) {

        PointerInfo pointerInfo = MouseInfo.getPointerInfo();
        Point current = pointerInfo.getLocation();
        Point[] points = divideLine(current, new Point(x, y));
        for (Point point : points) {

            robot.mouseMove(point.x, point.y);
            synchronized (point) {

                try {

                    point.wait(DIVIDE_WAIT);

                } catch (InterruptedException e) {}
            }
        }
    }


    private static Point[] divideLine(Point start, Point end) {

        double startX = start.x;
        double startY = start.y;
        double endX = end.x;
        double endY = end.y;

        double diffX = endX - startX;
        double diffY = endY - startY;

        /*
         * 線分の長さを取得する
         */
        double length = Math.sqrt((diffX * diffX) + (diffY * diffY));
        int count = (int)Math.round(length / DIVIDE_PX);
        Point[] points = new Point[count + 1];
        points[0] = new Point(start);
        for (int i = 1; i < count; i++) {

            int x = (int)Math.round(startX + (endX - startX) * i / count);
            int y = (int)Math.round(startY + (endY - startY) * i / count);
            points[i] = new Point(x, y);
        }

        points[count] = new Point(end);

        return points;
    }


    public static synchronized void mousePress(MouseButton mouseButton, ModifyKey modifyKey) {

        mousePress(getRobot(), mouseButton, modifyKey);
    }


    private static void mousePress(Robot robot, MouseButton mouseButton, ModifyKey modifyKey) {

        final int button = getButton(mouseButton);

        try {

            modifyKeyPress(robot, modifyKey);
            robot.mousePress(button);

        } finally {

            /*
             * タイマの設定 一定時間マウスキーがリリースされない場合、リリースする。
             */
            mouseReleaseTimer = new Timer();
            mouseReleaseTimer.schedule(new TimerTask() {

                @Override
                public void run() {

                    getRobot().mouseRelease(button);
                    mouseReleaseTimer = null;
                    LOG.debug("---------- Timeout Mouse released. ----------");
                }

            }, 10 * 1000);

            modifyKeyRelease(robot, modifyKey);
        }
    }


    public static synchronized void mouseRelease(MouseButton mouseButton, ModifyKey modifyKey) {

        mouseRelease(getRobot(), mouseButton, modifyKey);
    }


    private static void mouseRelease(Robot robot, MouseButton mouseButton, ModifyKey modifyKey) {

        if (null != mouseReleaseTimer) {

            mouseReleaseTimer.cancel();
            mouseReleaseTimer = null;
        }

        int button = getButton(mouseButton);

        try {

            modifyKeyPress(robot, modifyKey);
            robot.mouseRelease(button);

        } finally {

            modifyKeyRelease(robot, modifyKey);
        }
    }


    private static int getButton(MouseButton mouseButton) {

        int buttons = 0;

        if (mouseButton.isLeftButton()) {

            buttons = buttons | InputEvent.BUTTON1_MASK;
        }

        if (mouseButton.isCenterButton()) {

            buttons = buttons | InputEvent.BUTTON2_MASK;
        }

        if (mouseButton.isRightButton()) {

            buttons = buttons | InputEvent.BUTTON3_MASK;
        }

        return buttons;
    }


    public static synchronized void mouseClick(MouseButton mouseButton, ModifyKey modifyKey) {

        mouseClick(getRobot(), mouseButton, modifyKey);
    }


    private static void mouseClick(Robot robot, MouseButton mouseButton, ModifyKey modifyKey) {

        mousePress(robot, mouseButton, modifyKey);
        mouseRelease(robot, mouseButton, modifyKey);
    }


    public static synchronized void mouseWheel(int wheelAmt) {

        getRobot().mouseWheel(wheelAmt);
    }


    public static synchronized void keyPress(int jsKeycode, ModifyKey modifyKey) {

        keyPress(getRobot(), jsKeycode, modifyKey);
    }


    private static void keyPress(Robot robot, int jsKeycode, ModifyKey modifyKey) {

        int keycode = jsKeycode;

        if (convertKeycodeMap.containsKey(keycode)) {

            keycode = convertKeycodeMap.get(keycode);
        }

        try {

            modifyKeyPress(robot, modifyKey);
            robot.keyPress(keycode);

        } finally {

            modifyKeyRelease(robot, modifyKey);
        }
    }


    public static synchronized void keyRelease(int jsKeycode, ModifyKey modifyKey) {

        keyRelease(getRobot(), jsKeycode, modifyKey);
    }


    private static void keyRelease(Robot robot, int jsKeycode, ModifyKey modifyKey) {

        int keycode = jsKeycode;

        if (convertKeycodeMap.containsKey(keycode)) {

            keycode = convertKeycodeMap.get(keycode);
        }

        try {

            modifyKeyPress(robot, modifyKey);
            robot.keyRelease(keycode);

        } finally {

            modifyKeyRelease(robot, modifyKey);
        }
    }


    public static synchronized void keyType(int jsKeycode, ModifyKey modifyKey) {

        keyType(getRobot(), jsKeycode, modifyKey);
    }


    private static void keyType(Robot robot, int jsKeycode, ModifyKey modifyKey) {

        int keycode = jsKeycode;

        if (convertKeycodeMap.containsKey(keycode)) {

            keycode = convertKeycodeMap.get(keycode);
        }

        try {

            modifyKeyPress(robot, modifyKey);
            robot.keyPress(keycode);
            robot.keyRelease(keycode);

        } finally {

            modifyKeyRelease(robot, modifyKey);
        }
    }


    private static void modifyKeyPress(Robot robot, ModifyKey modifyKey) {

        if (null == modifyKey) {

            return;
        }

        if (modifyKey.isCtrlKey()) {

            robot.keyPress(KeyEvent.VK_CONTROL);
        }

        if (modifyKey.isShiftKey()) {

            robot.keyPress(KeyEvent.VK_SHIFT);
        }

        if (modifyKey.isAltKey()) {

            robot.keyPress(KeyEvent.VK_ALT);
        }

        if (modifyKey.isMetaKey()) {

            robot.keyPress(KeyEvent.VK_META);
        }

    }


    private static void modifyKeyRelease(Robot robot, ModifyKey modifyKey) {

        if (null == modifyKey) {

            return;
        }

        if (modifyKey.isCtrlKey()) {

            robot.keyRelease(KeyEvent.VK_CONTROL);
        }

        if (modifyKey.isShiftKey()) {

            robot.keyRelease(KeyEvent.VK_SHIFT);
        }

        if (modifyKey.isAltKey()) {

            robot.keyRelease(KeyEvent.VK_ALT);
        }

        if (modifyKey.isMetaKey()) {

            robot.keyRelease(KeyEvent.VK_META);
        }
    }


    public static synchronized void typeText(String text) {

        Robot robot = getRobot();

        for (int i = 0; i < text.length(); i++) {

            typeCharKey(robot, text.charAt(i));
        }
    }


    /**
     * 指定されたキャラクタのキーを押して離します。
     *
     * @param keycode
     */
    private static void typeCharKey(Robot robot, char c) {

        KeyChar keyChar = getKeyCode(c);
        if (null == keyChar) {

            return;
        }

        if (keyChar.isShift()) {

            keyPress(robot, KeyEvent.VK_SHIFT, null);
        }

        keyType(robot, keyChar.getKeyCode(), null);

        if (keyChar.isShift()) {

            keyRelease(robot, KeyEvent.VK_SHIFT, null);
        }
    }


    private static KeyChar getKeyCode(char c) {

        return KeyChar.getKeyChar(c);
    }


    public static synchronized void screenCapture(int x, int y, int width, int height, String name) {

        screenCapture(getRobot(), x, y, width, height, name);
    }


    private static void screenCapture(Robot robot, int x, int y, int width, int height, String name) {

        /* \/:*?"<>| */
        name = name.replace('\\', '￥');
        name = name.replace('/', '／');
        name = name.replace(':', '：');
        name = name.replace('*', '＊');
        name = name.replace('?', '？');
        name = name.replace('"', '”');
        name = name.replace('<', '＜');
        name = name.replace('>', '＞');
        name = name.replace('|', '｜');

        Rectangle screenRect = new Rectangle(x, y, width, height);
        BufferedImage image = robot.createScreenCapture(screenRect);
        String dirName = getParameter("capture.save.dirname");
        String home = System.getProperty("user.home");
        File dir = new File(home, dirName);
        dir.mkdirs();
        SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy.MM.dd_HH-mm-ss.SSS");
        File path = new File(dir, dateFormat.format(new Date()) + "_" + name + ".png");

        LOG.debug(path.getAbsolutePath());

        try {

            ImageIO.write(image, "png", path);

        } catch (IOException e) {

            throw new RobotException(path.getAbsolutePath(), e);
        }
    }


    private static String getParameter(String key) {

        return "capture";
    }
}
